/*
 *  Roadnav
 *  App.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Application code.
///
/// Contains the entry point for the application.
///
//////////////////////////////////////////////////////////////////////////////
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _MSC_VER
#pragma warning(disable: 4786)
#endif

#include <wx/wx.h>
#include <wx/image.h>
#include <wx/config.h>
#include <wx/fs_zip.h>
#include <wx/fs_inet.h>
#include <wx/socket.h>
#include <wx/stdpaths.h>

#include "App.h"
#include "Frame.h"
#include "libroadnav/Map.h"
#include "TTS.h"
#include "libroadnav/URLs.h"
#include "libroadnav/MapSupport.h"
#include "Support.h"
#include "libroadnav/DownloadFiles.h"
#include "libroadnav/Debug.h"

#include "splash.xpm"

IMPLEMENT_APP(MapApp)

#if !LIBROADNAV_CHECK_VERSION(0, 20)
#error Requires at least version 0.20 of LibRoadnav
#endif

#if !wxCHECK_VERSION(2, 6, 2)
#error Requires at least version 2.6.2 of wxWidgets
#endif

//////////////////////////////////////////////////////////////////////////////
/// 
/// \brief Show the splash screen
///
//////////////////////////////////////////////////////////////////////////////
void MapApp::ShowSplash()
{
	wxBitmap bmpSplash(splash_xpm);
	
	m_wndSplash = new wxSplashScreen(
						bmpSplash,
						wxSPLASH_CENTRE_ON_SCREEN | wxSPLASH_NO_TIMEOUT,
						0,
						NULL,
						-1,
						wxDefaultPosition,
						wxDefaultSize,
						wxSIMPLE_BORDER | wxFRAME_NO_TASKBAR
						);
	
	m_wndSplash->SetTitle(wxT("Roadnav"));

	wxYield();
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Hide the splash screen
///
//////////////////////////////////////////////////////////////////////////////
void MapApp::HideSplash()
{
	delete m_wndSplash;
}

#if defined(_MSC_VER)
typedef BOOL (WINAPI * FHeapSetInformation)(HANDLE, HEAP_INFORMATION_CLASS, PVOID, SIZE_T);
#endif

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Initialization code
///
//////////////////////////////////////////////////////////////////////////////
bool MapApp::OnInit()
{
	wxString instance = GetTempDir() +  wxT("/roadnav-") + wxGetUserId();
	wxConfigBase * pConfig = wxConfigBase::Get();

	// checks to see if we're already running, and go away if so
	m_checker = new wxSingleInstanceChecker( wxT("roadnav-") + wxGetUserId());
	if (m_checker->IsAnotherRunning())
	{
		// make a connection to tell the other side of the app that we're here, don't need to transfer any data
		wxClient * client = new wxClient();
		wxConnectionBase * connection = client->MakeConnection(wxT("localhost"), instance, wxT("showroadnav"));
		if (connection)
		{
			connection->Disconnect();
			delete connection;
		}

		delete client;

		return false;
	}
    
	// otherwise, create a server to receive notifications
	m_instanceServer = new MapAppInstanceServer(this);
	m_instanceServer->Create(instance);

#if defined(_MSC_VER)
	if (!IsDebuggerPresent())
	{
		HMODULE hKernel32;

		hKernel32 = LoadLibrary(_T("kernel32.dll"));

		if (hKernel32)
		{
			FHeapSetInformation fHeapSetInformation;

			fHeapSetInformation = (FHeapSetInformation) GetProcAddress(hKernel32, "HeapSetInformation");
			if (fHeapSetInformation)
			{
				// this improves Windows memory performance quite a bit
				ULONG ulEnableLFH = 2; //try 1 else
				BOOL bSuccess = fHeapSetInformation((PVOID) _get_heap_handle(),
								HeapCompatibilityInformation,
								&ulEnableLFH, 
								sizeof(ulEnableLFH));
			}
		}
	}
#endif
        
	SetVendorName(wxT(""));
	SetAppName(wxT("roadnav"));

	pConfig->SetRecordDefaults();
	pConfig->SetExpandEnvVars(false);
	
	wxSocketBase::Initialize();

	if (!wxApp::OnInit())
		return false;

	////////////////////////////////////////////////////////////////
	// For some reason, if a wxScreenDC is not instantiated, it's
	// not possible to focus the Roadnav window under OSX.
	////////////////////////////////////////////////////////////////
	wxScreenDC dcScreen;

	////////////////////////////////////////////////////////////////
	// Initialize stuff
	////////////////////////////////////////////////////////////////
	wxInitAllImageHandlers();

	wxFileSystem::AddHandler(new wxZipFSHandler);
	wxFileSystem::AddHandler(new wxInternetFSHandler);

#ifdef LIBROADNAV_DEBUG
	if (pConfig->Read(wxT("GenerateDebugLog"), (long) 0))
		LibRoadnavDebugEnable();
#endif

	////////////////////////////////////////////////////////////////
	// Splash screen on, if debug mode not used
	////////////////////////////////////////////////////////////////

#ifndef _DEBUG
	ShowSplash();
#endif
	
	////////////////////////////////////////////////////////////////
	// Common data path
	////////////////////////////////////////////////////////////////

	SetCommonDataPath(RoadnavCommonDirectory());

	////////////////////////////////////////////////////////////////
	// Determine local path
	////////////////////////////////////////////////////////////////
	wxFileName fnLocalBase;
	wxString strRegLocalDataDir;
	
	strRegLocalDataDir = pConfig->Read(wxT("LocalDataDirectory"), wxT(""));

	if (strRegLocalDataDir != wxT(""))
	{
		fnLocalBase.AssignDir(strRegLocalDataDir);
	}
	else
	{
		wxStandardPathsBase & p = wxStandardPaths::Get();
		fnLocalBase.AssignDir(p.GetUserDataDir() + wxT("data"));
	}

	if (!fnLocalBase.DirExists())
		fnLocalBase.Mkdir(0755);

	////////////////////////////////////////////////////////////////
	// Local data path
	////////////////////////////////////////////////////////////////
	wxFileName fnLocal;
	
	fnLocal = fnLocalBase;
	fnLocal.AppendDir(wxT("compiled_maps"));

	SetLocalDataPath(fnLocal.GetFullPath());

	//////////////////////////////////////////////////////////////////////////////
	// Download cache directory
	//////////////////////////////////////////////////////////////////////////////
	wxFileName fnCache;

	fnCache = fnLocalBase;
	fnCache.AppendDir(wxT("http_cache"));

	SetDownloadCacheDirectory(fnCache.GetFullPath());

	//////////////////////////////////////////////////////////////////////////////
	// Proxy settings
	//////////////////////////////////////////////////////////////////////////////
	ConfigureProxy();

	////////////////////////////////////////////////////////////////
	// Eliminate splash screen
	////////////////////////////////////////////////////////////////

#ifndef _DEBUG
	HideSplash();
#endif

	////////////////////////////////////////////////////////////////
	// Bring up the main window
	////////////////////////////////////////////////////////////////

	pfMain = new MapFrame(wxT("Roadnav"));
	pfMain->Show(TRUE);
	SetTopWindow(pfMain);
	if (m_bFullScreen)
		pfMain->SetFullScreenMode(m_bFullScreen);
	
	return TRUE;	
}


int MapApp::OnExit()
{
	wxApp::OnExit();

    if (m_checker)
        delete m_checker;
	if (m_instanceServer)
		delete m_instanceServer;
    return 0;
}

void MapApp::OnInitCmdLine(wxCmdLineParser & parser)
{
	wxString strLogo;
	
	wxApp::OnInitCmdLine(parser);
	
	strLogo = wxT("Roadnav ") wxT(VERSION) wxT("\n");
	strLogo += wxT("\nCopyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>\n");
	strLogo += wxT("\n");

	strLogo += wxT("This program is free software; you can redistribute it and/or\n");
	strLogo += wxT("modify it under the terms of version 2 of the GNU General Public License\n");
	strLogo += wxT("as published by the Free Software Foundation.\n");
	strLogo += wxT("\n");
	strLogo += wxT("This program is distributed in the hope that it will be useful,\n");
	strLogo += wxT("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
	strLogo += wxT("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
	strLogo += wxT("GNU General Public License for more details.\n");
	strLogo += wxT("\n");
	strLogo += wxT("You should have received a copy of the GNU General Public License\n");
	strLogo += wxT("along with this program (COPYING in the application folder);\n");
	strLogo += wxT("if not, write to the Free Software Foundation, Inc.,\n");
	strLogo += wxT("51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA\n");
	
	parser.SetLogo(strLogo);
	
	parser.AddSwitch(wxT("f"), wxT("fullscreen"), wxT("Start in full screen mode"));
#ifdef LIBROADNAV_DEBUG
	parser.AddSwitch(wxT("d"), wxT("debug"), wxT("Enable debugging"));
#endif
#ifdef USE_SCRIPTING
	parser.AddOption(wxT("c"), wxT("command"), wxT("Send scripting command to TCP server"));
#endif
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Called to parse command line arguments.
///
//////////////////////////////////////////////////////////////////////////////
bool MapApp::OnCmdLineParsed(wxCmdLineParser & parser)
{
	bool bContinue = true;
	wxString strArg;

	m_bFullScreen = false;
	
	if (!wxApp::OnCmdLineParsed(parser))
		bContinue = false;
	
	if (parser.Found(wxT("f")))
		m_bFullScreen = true;

#ifdef LIBROADNAV_DEBUG
	if (parser.Found(wxT("d")))
		LibRoadnavDebugEnable();
#endif

#ifdef USE_SCRIPTING
	if (parser.Found(wxT("c"), &strArg))
	{
		int iServerPort = -1;
		
		if (wxConfigBase::Get()->Read(wxT("ScriptingTCPServerEnabled"), (long) 0))
			iServerPort = wxConfigBase::Get()->Read(wxT("ScriptingTCPServerPortNumber"), DEFAULT_SCRIPTING_TCP_PORT_NUMBER);

		if (iServerPort >= 0)
		{
			wxSocketClient sockRoadnav;
			wxIPV4address addrRoadnav;

			addrRoadnav.Hostname(wxT("127.0.0.1"));
			addrRoadnav.Service(iServerPort);		
			
			sockRoadnav.Connect(addrRoadnav, true);

			if (sockRoadnav.IsConnected())
			{
				char * pszMsg;
				
				pszMsg = new char [strArg.Length() + 3];
				strcpy(pszMsg, wx_c_str(strArg));
				strcat(pszMsg, "\r\n");
				
				sockRoadnav.Write(pszMsg, strlen(pszMsg));
				sockRoadnav.Close();
				
				delete pszMsg;
			}
			else
			{
				// Roadnav not running - silently ignore
			}
		}
		else
		{
			wxMessageBox(	wxT("The scripting TCP server must be abled to use the -c command line option."), 
							wxT("Error"), 
							wxOK | wxICON_ERROR);
		}

		bContinue = false;
	}
#endif
		
	return bContinue;
}


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Platform-independent way to tell the application to give the
/// main window focus
///
//////////////////////////////////////////////////////////////////////////////
void MapApp::ShowFrame()
{
	pfMain->Maximize(false);
	pfMain->SetFocus();
}

wxConnectionBase * MapAppInstanceServer::OnAcceptConnection(const wxString& topic)
{
	if (topic == wxT("showroadnav"))
		m_app->ShowFrame();
	return new wxConnection();
}

