/*
 *  Roadnav
 *  CyborgNavigator.cpp
 *
 *  Copyright (C) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  Optimized of python implementation originally written by 
 *  Randall Munroe (xkcd.com), 2008-05-17
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// This is a class for reading aloud cardinal directions to a specified 
/// destination
///
//////////////////////////////////////////////////////////////////////////////

#include <wx/wx.h>
#include <wx/config.h>
#include <math.h>

#include "CyborgNavigator.h"

#define TURNED_DEGREES 45.0

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Cyborg Navigator constructor. Specify a config class and a
/// destination latitude/longitude for it to navigate to
///
//////////////////////////////////////////////////////////////////////////////
CyborgNavigator::CyborgNavigator(double destination_lat, double destination_lon) :
	m_avgSpeed_total(0), m_avgSpeed_last(0), m_avgSpeed_n(0),
	m_destination_lat(destination_lat), m_destination_lon(destination_lon),
	m_last_heading(1024)
{
	m_words.Add(_("North"));
	m_words.Add(_("North-East"));
	m_words.Add(_("East"));
	m_words.Add(_("South-East"));
	m_words.Add(_("South"));
	m_words.Add(_("South-West"));
	m_words.Add(_("West"));
	m_words.Add(_("North-West"));
	
	ReloadSettings();
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Call this after preferences have been changed
///
//////////////////////////////////////////////////////////////////////////////
void CyborgNavigator::ReloadSettings()
{
	long tmp;
	wxConfigBase * pConfig = wxConfigBase::Get();

	pConfig->Read(wxT("Units"),&tmp,0);
	m_units = (EUnits)tmp;

	// various timeout values
	pConfig->Read(wxT("CyborgNavDistanceTimeout"), &tmp, 20);
	m_distance_timeout = tmp;

	pConfig->Read(wxT("CyborgNavDirectionTimeout"), &tmp, 10);
	m_direction_timeout = tmp;
		
	pConfig->Read(wxT("CyborgNavGpsFixTimeout"), &tmp, 15);
	m_gps_fix_speech_timeout = tmp;

	// setup defaults so that it starts speaking again
	m_last_distance_speech = m_last_direction_change_speech = 
	m_last_direction_speech  =	m_last_gps_fix_speech = 0;
}


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Set the destination latitude and longitude
///
//////////////////////////////////////////////////////////////////////////////
void CyborgNavigator::SetDestination(double lat, double lon)
{
	m_destination_lat = lat;
	m_destination_lon = lon;
	
	// setup defaults so that it starts speaking again
	m_last_distance_speech = m_last_direction_change_speech = 
	m_last_direction_speech  =	m_last_gps_fix_speech = 0;
}


//////////////////////////////////////////////////////////////////////////////
///
/// \brief This coordinates the actual "navigation" that this class performs,
/// and ideally it should be called every second with a new GPS fix. Though,
/// you could call it every min(m_distance_timeout, m_direction_timeout)
/// seconds and be fine. 
///
//////////////////////////////////////////////////////////////////////////////
void CyborgNavigator::Navigate(const wxGPSEvent &evGPS, ITTS * tts)
{
	bool speak_direction = false, speak_distance = false;

	// get the current timestamp
	time_t current_time = wxDateTime::GetTimeNow();

	// calculate a running average for speed
	CalculateAverageSpeed(evGPS.m_fSpeed);

	// see if we have a fix
	if (!evGPS.m_bLocked) 
	{
		if (current_time > m_last_gps_fix_speech + m_gps_fix_speech_timeout)
		{
			m_last_gps_fix_speech = current_time;
			tts->Speak(_("No GPS fix"));
		}
		return;
	}

	// speak the distance?
	if (current_time > m_last_distance_speech + m_distance_timeout)
	{
		m_last_distance_speech = current_time;
		speak_distance = true;
	}

	// determine if the user changed direction, but don't speak too often
	if (current_time > m_last_direction_speech + m_direction_timeout ||
		(current_time > m_last_direction_change_speech + 2 && 
		 (evGPS.m_fHeading > m_last_heading + TURNED_DEGREES ||
		  evGPS.m_fHeading < m_last_heading - TURNED_DEGREES)))
	{
		m_last_direction_speech = current_time;
		m_last_direction_change_speech = current_time;
		speak_direction = true;
	}


	// speak to me!
	if (speak_direction || speak_distance)
	{
		double lat = evGPS.m_pt.GetLat();
		double lon = evGPS.m_pt.GetLong();
		double vx, vy, target_heading, target_distance, target_oclock;
		wxString speak_this;

		Vectorize(lon, lat, m_destination_lon, m_destination_lat, vx, vy);

		target_heading = Angle(vx, vy);
		target_oclock = GetOffset(evGPS.m_fHeading, target_heading);

		if (speak_distance)
		{
			// find the distance in KM
			target_distance = sqrt(vx*vx + vy*vy);

			//speak something to this effect:
			// Target 30 kilometers North-West at 6 oclock... eetee eigh 2 Hours 17 minutes
			speak_this = wxT("Target ");

			if (m_units == USUnits)
			{
				// do this conversion since we need it anyways
				target_distance *= 0.621371192;

				if (target_distance > 0.5)
					speak_this += wxString::Format(wxT("%.2f miles "),target_distance);
				else
					speak_this += wxString::Format(wxT("%d feet "), (int)(target_distance * 5280));
			} 
			else {
				// ok, speak in metric units
				if (target_distance > 0.5)
					speak_this += wxString::Format(wxT("%.2f kilometers "),target_distance);
				else
					speak_this += wxString::Format(wxT("%d meters "), (int)(target_distance * 1000));

				// do this conversion last, so we don't have another calculation to do
				target_distance *= 0.621371192;
			}

			// give a direction
			speak_this += GetDirectionalWord(target_heading);
			speak_this += wxString::Format(wxT(" at %d oh clock... ") , Offset2OClock(target_oclock));

			// only give ETA if we're moving
			if (m_avgSpeed_last*5280 > 10)
			{
				double eta = (target_distance / m_avgSpeed_last) * 3600; 
				speak_this += wxT("eetee eigh ");
				
				int hours = eta / 3600;
				int minutes = fmod(eta / 60, 60);

				if (hours)
					speak_this += wxString::Format(wxT("%d"), hours) + (hours > 1 ? wxT(" hours ") : wxT(" hour "));
				if (minutes)
					speak_this += wxString::Format(wxT("%d"), minutes) +(minutes > 1 ? wxT(" minutes ") : wxT(" minute "));
				else if (!hours)
					speak_this += wxString::Format(wxT("%d"), (int)eta) +(((int)eta) != 1 ? wxT(" seconds ") : wxT(" second "));
			} 
		} 

		// dont give a seperate direction if we already did it above
		else if (speak_direction)
			speak_this += wxString::Format(wxT("Target %d oh clock") , Offset2OClock(target_oclock));
			

		// ok, do the speaking part here
		tts->Speak(speak_this);
	} 

	m_last_heading = evGPS.m_fHeading;
}

//////////////////////////////////////////////////////////////////////////////
///
/// Math Routines
///
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Returns a vector in x and y
///
//////////////////////////////////////////////////////////////////////////////
void CyborgNavigator::Vectorize(double x1, double y1, double x2, double y2, double &x, double &y)
{
	y = y2 - y1;
	x = x2 - x1;
	
	// kilometers
	y *= 111.0;
	x *= 111.0;
	
	x *= abs(cos((y1/360)* 2 * PI));
}


inline
double CyborgNavigator::Angle(double x, double y)
{
	return fmod( 450 - (atan2(y, x) * (180.0 / PI)),360.0);
}

inline 
double CyborgNavigator::GetOffset(double heading, double angle)
{
    double a = fmod((angle+180.0)-heading,360.0)-180.0;
	if (a < 0)
		a += 360;
	return a;
}

inline 
int CyborgNavigator::Offset2OClock(double angle)
{
    int clock = (int)(fmod(angle+15.0, 360.0)/30.0);
	if (!clock)
		clock = 12;
	return clock;
}

wxString& CyborgNavigator::GetDirectionalWord(double angle)
{
	angle += 22.5;
	return m_words[((((int)angle) % 360 )/45) % 8];
}

// calculates average speed, tries to smooth out things 
// for ETA calculations. If we wanted it to be strictly
// correct, there would be some kind of time basis.. meh
void CyborgNavigator::CalculateAverageSpeed(double speed)
{
	// adjust this parameter to make it fall/rise faster/slower
	if (m_avgSpeed_n < 10)
		m_avgSpeed_n += 1;
	else
		m_avgSpeed_total -= (m_avgSpeed_last * 1.005);
	
	m_avgSpeed_total += speed;
	m_avgSpeed_last = m_avgSpeed_total / m_avgSpeed_n;
}
