/*
 *  Roadnav
 *  OptionCustom.h 
 *
 *  Copyright (c) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Definition of custom options for special cases -- ideally, these special
/// cases should be few and far between. 
///
//////////////////////////////////////////////////////////////////////////////

#ifndef OPTION_CUSTOM_H
#define OPTION_CUSTOM_H

#include <libroadnav/URLs.h>
#include <libroadnav/Map.h>
#include <libroadnav/DownloadFiles.h>
#include "SerialIO.h"


class MapUrlCustomStringOption : public wxStringOption {
	public:
		MapUrlCustomStringOption(int index) {
			m_description = GetURLName(index);
			m_defaultValue = GetURL(m_description);
		};

		bool DoSave() {
			SetURL(m_description, m_defaultValue);
			return true;
		};
};

class MapUrlCustomOptionGroup : public wxOptionGroup {
	public:
		MapUrlCustomOptionGroup(const wxString& name) : wxOptionGroup(name) {
			int i = 0;
			while (GetURLName(i) != wxT(""))
				Add(new MapUrlCustomStringOption(i++));
		};
};

class serialPortComboBoxOption : public wxComboBoxOption {
	public:
		serialPortComboBoxOption(const wxString& configValueName, const wxString& description, const wxString& defaultValue) :
		  		wxComboBoxOption(configValueName, description, defaultValue, false)
		{};

		void DoDisplay(wxOptionSizer * sizer, wxWindow * parent);

};

class ttsTypeOption : public wxEvtHandler, public wxComboBoxOption {
	public:
		ttsTypeOption(const wxString& configValueName, const wxString& description, const wxString& defaultValue);
		void DoDisplay(wxOptionSizer * sizer, wxWindow * parent);
		void OnTest(wxCommandEvent& event);
		bool Enable(bool enable){ 
			return m_testButton->Enable(enable) &&
				m_testText->Enable(enable) &&
				this->wxComboBoxOption::Enable(enable);
		};

	private:
		wxButton * m_testButton;
		wxTextCtrl * m_testText;	
};

// option to show a button to clear the map data
class clearMapDataOption : public wxEvtHandler, public wxOption {
	public:
		clearMapDataOption() : wxOption() {};

		void DoDisplay(wxOptionSizer * sizer, wxWindow * parent) 
		{
			sizer->AddStretchSpacer();
			sizer->Add(m_button = new wxButton(parent, wxID_ANY, _("Clear Map Data")));

			m_button->Connect(m_button->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
				wxCommandEventHandler(clearMapDataOption::OnClick));
		};

		void OnClick(wxCommandEvent& event)
		{
			if (wxMessageBox(_("This will erase all compiled map data. It is recommended you restart Roadnav after performing this operation. Are you sure you want to continue?"), _("Confirm"), wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION) == wxYES)
			{
				ClearCompiledMapData();
				wxMessageBox(_("Map data cleared. Please restart Roadnav."), _("Information"), wxOK | wxICON_INFORMATION);
			}
		};

		void OnWindowDestroy() { };
		bool DoSave() { return true; };
		bool Enable(bool enable) { return m_button->Enable(enable);  };

	private:
		wxButton * m_button;
};

// option to show a button to clear the cache
class clearCacheOption : public wxEvtHandler, public wxOption {
	public:
		clearCacheOption() : wxOption() {};

		void DoDisplay(wxOptionSizer * sizer, wxWindow * parent) 
		{
			sizer->AddStretchSpacer();
			sizer->Add(m_button = new wxButton(parent, wxID_ANY, _("Clear Download Cache")));

			m_button->Connect(m_button->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
				wxCommandEventHandler(clearMapDataOption::OnClick));
		};

		void OnClick(wxCommandEvent& event)
		{
			if (wxMessageBox(_("This will erase all cached downloads. It will not affect the data used to draw the maps. Are you sure you want to continue?"), _("Confirm"), wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION) == wxYES)
			{
				ClearDownloadCache();
				wxMessageBox(_("Cache cleared."), _("Information"), wxOK | wxICON_INFORMATION);
			}
		};

		void OnWindowDestroy() { };
		bool DoSave() { return true; };
		bool Enable(bool enable) { return m_button->Enable(enable);  };

	private:
		wxButton * m_button;
};

class skinComboBoxOption : public wxComboBoxOption {
	public:
		skinComboBoxOption(const wxString& configValueName, const wxString& description, const wxString& defaultValue);
};


#endif

