/*
 *  OptionsLaunch.cpp
 *
 *  Copyright (c) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Defines project-specific menus and such, and launches the wxOptionsDialog
/// window. Modify this file when you want to add options.
///
/// The biggest problem with the way this is currently implemented is that
/// everything is totally centralized here and this is *mostly* independent, 
/// so you need to also implement read/write support into other parts of the
/// program, which raises the possibility of them being out of sync. However,
/// I can't think of a decent way to solve this problem, even if the entire
/// program was reorganized
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "wxOptionsDialog.h"
#include "wxOptionMacros.h"

#include "OptionCustom.h"
#include "OptionThemes.h"

#include "OptionsLCDproc.h"

// roadnav headers

#include "SerialIO.h"
#include "Support.h"
#include "GPSMonitorThread.h"


int ShowOptionsDialog( wxWindow * parent, IMapData * pMapData, ThemeManager * pThemeManager )
{
    // define an option group
    OPTIONS_START(rootOptionGroup)

    DEF_OPTION_GROUP(_("General"))
        
        DEF_INT_OPTION(wxT("HomeZipCode"), _("Home Zip Code"), 0)
        
        DEF_BOOL_OPTION(wxT("EnableExitCommand"), _("Enable Exit Command"), false)
        DEF_STRING_OPTION(wxT("ExitCommand"), _("Exit Command"), GetDefaultShutdownCommand())
        
        DEF_BOOL_OPTION(wxT("AlwaysUseAdvancedWaypointsDialog"), _("Always Use Advanced Waypoint Dialog Box"), false)

		DEF_OPTION(new clearCacheOption())
		DEF_OPTION(new clearMapDataOption())
    END_OPTION_GROUP

    DEF_OPTION_GROUP(_("Skins"))
        // day skin, enumerate options (same skin)
		DEF_OPTION(new skinComboBoxOption(wxT("SkinPath-Day"), _("Daytime Skin"), wxT("Default-Day")))
        DEF_OPTION(new skinComboBoxOption(wxT("SkinPath-Night"), _("Nighttime Skin"), wxT("Default-Night")))
    END_OPTION_GROUP


    DEF_OPTION_GROUP(_("GPS"))
    
        DEF_OPTION_GROUP(_("GPS Type"))

            // we *could* get default from GPSMonitorThread::GetGPSType(), however it just
		    // reads a value from the config... so do the same thing
		    DEF_COMBOBOX_INDEXED_ITEMS(wxT("GPSType"), _("GPS type"), wxT("Auto Detect"))
    		
			    DEF_COMBOBOX_PAIR(GPSTypeNone, _("None"))
			    DEF_COMBOBOX_PAIR(GPSTypeAutoDetect, _("Auto Detect"))
    #ifdef ROADNAV_SERIAL_SUPPORT
			    DEF_COMBOBOX_PAIR(GPSTypeSerial, _("Serial/USB"))
    #endif
    #ifdef HAVE_LIBGPS
			    DEF_COMBOBOX_PAIR(GPSTypeGPSD, _("GPSD"))
    #endif
    #ifdef HAVE_SIMULATED_GPS_SUPPORT
			    DEF_COMBOBOX_PAIR(GPSTypeSimulated, _("Simulated"))
    #endif
		    END_COMBOBOX_ITEMS
       

    #ifdef ROADNAV_SERIAL_SUPPORT
            
		    DEF_SUB_OPTION_GROUP(_("Serial Configuration"))

			    DEF_OPTION(new serialPortComboBoxOption(wxT("GPSSerialPort"),_("Serial port"), wxT("")))

			    DEF_COMBOBOX_INDEXED_ITEMS(wxT("GPSBaudRate"), _("Baud Rate"), wxT("4800"))
				    DEF_COMBOBOX_PAIR(1200,  wxT("1200"))
				    DEF_COMBOBOX_PAIR(2400,  wxT("2400"))
				    DEF_COMBOBOX_PAIR(4800,  wxT("4800"))
				    DEF_COMBOBOX_PAIR(9600,  wxT("9600"))
				    DEF_COMBOBOX_PAIR(19200, wxT("19200"))
				    DEF_COMBOBOX_PAIR(38400, wxT("38400"))
				    DEF_COMBOBOX_PAIR(57600, wxT("57600"))
				    DEF_COMBOBOX_PAIR(115200,wxT("115200"))
			    END_COMBOBOX_ITEMS
            
			    DEF_STRING_OPTION(wxT("GPSSerialInitString"), _("Serial init string"), wxEmptyString)
    			
		    END_SUB_OPTION_GROUP
    #endif

    #ifdef HAVE_LIBGPS

		    DEF_SUB_OPTION_GROUP(_("GPSD Config"))

			    DEF_STRING_OPTION(wxT("gpsdHost"), _("gpsd Host"), wxT("localhost"))
			    DEF_INT_OPTION(wxT("gpsdPort"), _("gpsd Port"), 2947)
                
			    DEF_BOOL_OPTION(wxT("gpsdCompatibilityCheck"), _("Check gpsd Server Compatibility"), true)
			    DEF_BOOL_OPTION(wxT("gpsdBufferingPolicy"), _("gpsd Buffering (Reduces Jitter)"), false)

		    END_SUB_OPTION_GROUP
    #endif
        END_OPTION_GROUP

        DEF_OPTION_GROUP(_("Display"))
		    DEF_BOOL_OPTION(wxT("GPS Callout Box"), _("GPS Callout Box"), false)
		    DEF_BOOL_OPTION(wxT("GPS Small Label"), _("GPS Small Label"), true)
		    DEF_BOOL_OPTION(wxT("GPS Arrow"), _("GPS Arrow"), true)
		    DEF_FILE_OPTION(wxT("GPS Icon"), _("GPS Icon"), wxEmptyString, wxT("Images|*.bmp;*.png;*.jpg;*.jpeg;*.gif;*.pcx;*.pnm;*.tif;*.tiff;*.xpm;*.ico"), _("Select GPS icon..."))
		    DEF_BOOL_OPTION_INVERSE(wxT("NorthAlwaysUp"), _("Rotate Map to Match Heading"), true)
		    DEF_BOOL_OPTION(wxT("KeepGPSCentered"), _("Keep GPS Centered"), true)
		    DEF_BOOL_OPTION(wxT("RoadTypeZooming"), _("Zoom Based on Road Type"), false )
            DEF_BOOL_OPTION(wxT("ShowPopupGPSErrors"), _("Show GPS errors in message box"), false)
        END_OPTION_GROUP

    END_OPTION_GROUP


    DEF_OPTION_GROUP(_("Maps"))

        DEF_GROWABLE_THING(new MapUrlCustomOptionGroup(_("Download URLs")))
		END_GROWABLE_THING

        DEF_OPTION_GROUP(_("Map Themes"))
            DEF_OPTION(new mapThemeOption(pMapData, pThemeManager))
        END_OPTION_GROUP

        DEF_OPTION_GROUP(_("Map Options"))

            DEF_BOOL_OPTION(wxT("AutoDownloadMaps"), _("Automatically download maps"), true)
            DEF_BOOL_OPTION( wxT("CacheDownloads"), _("Cache downloads"), true)
            DEF_DIRECTORY_OPTION(wxT("LocalDataDirectory"), _("Map Directory (blank = default)"), wxEmptyString, wxT("Select map directory (optional)"))

            DEF_DOUBLE_OPTION_HAS_MINMAX(wxT("DrawingScaleFactor"), _("Zoom Maps"), 1.0, 1e-5, 10.0)
            DEF_STRING_OPTION(wxT("SpeedControlledDetailLevelCutoffs"), _("Speed Controlled Zooming Cutoffs (mph)"), wxT("25, 50"))
        END_OPTION_GROUP

#ifdef USE_OPENSTREETMAP
        DEF_OPTION_GROUP(_("OpenStreetMap"))
            DEF_STRING_OPTION(wxT("OSMUsername"), _("Username"), wxEmptyString)
            DEF_PWD_OPTION(wxT("OSMPassword"), _("Password"), wxEmptyString)
		END_OPTION_GROUP
#endif

        DEF_OPTION_GROUP(_("Units"))
            DEF_COMBOBOX_ITEMS(wxT("PositionFormat"), _("Position Format"), POSITION_FORMAT_3)
			    DEF_COMBOBOX_ITEM(POSITION_FORMAT_1)
			    DEF_COMBOBOX_ITEM(POSITION_FORMAT_2)
			    DEF_COMBOBOX_ITEM(POSITION_FORMAT_3)
		    END_COMBOBOX_ITEMS

		    DEF_COMBOBOX_INDEXED_ITEMS(wxT("Units"), _("Units"), wxT("US (English)"))
			    DEF_COMBOBOX_ITEM(wxT("US (English)"))
			    DEF_COMBOBOX_ITEM(wxT("SI (metric)"))
		    END_COMBOBOX_ITEMS
        END_OPTION_GROUP

	END_OPTION_GROUP
    
	DEF_OPTION_GROUP(_("Navigation"))

		DEF_OPTION_GROUP(_("Cyborg Nav"))
			DEF_INT_OPTION(wxT("CyborgNavDistanceTimeout"), _("Repeat distance every N seconds"), 20)
			DEF_INT_OPTION(wxT("CyborgNavDirectionTimeout"), _("Repeat direction every N seconds"), 10)
			DEF_INT_OPTION(wxT("CyborgNavGpsFixTimeout"), _("Repeat loss of GPS fix every N seconds"), 15)
		END_OPTION_GROUP

		DEF_OPTION_GROUP(_("Text to speech"))
	    
		   DEF_BOOL_OPTION_GROUP(wxT("LiveDirectionsTTS"), _("Verbalize directions"), true, boolStoreNormal)
				DEF_OPTION(new ttsTypeOption(wxT("TTSType"), _("TTS Type"), wxT("")))
			END_BOOL_OPTION_GROUP

		END_OPTION_GROUP

		DEF_OPTION_GROUP(_("Routing"))
			DEF_NULL_OPTION(_("Move sliders to the left to have routing avoid a particular type of road, or right to prefer that type of road."))

			DEF_SLIDER_DOUBLE_OPTION(wxT("PreferSmallRoads"), _("Small Roads"), 1.0, 0.1, 6.0, 2)
			DEF_SLIDER_DOUBLE_OPTION(wxT("PreferLargeRoads"), _("Large Roads"), 1.44, 0.1, 6.0, 2)
			DEF_SLIDER_DOUBLE_OPTION(wxT("PreferHighways"), _("Highways"), 1.54, 0.1, 6.0, 2)
        END_OPTION_GROUP
            

	END_OPTION_GROUP

    
    DEF_OPTION_GROUP(_("Advanced"))

#ifdef USE_SCRIPTING
        DEF_OPTION_GROUP(_("Scripting"))
            DEF_BOOL_OPTION(wxT("ScriptingTCPServerEnabled"), _("TCP Server Enabled"), false)
            DEF_BOOL_OPTION(wxT("ScriptingTCPServerAcceptOnlyLocal"), _("Accept TCP Connections Only From This Computer"), true)
            //DEF_INT_OPTION(wxT("ScriptingTCPServerPortNumber"), _("TCP Server Port Number"), DEFAULT_SCRIPTING_TCP_PORT_NUMBER)
        
            DEF_BOOL_OPTION(wxT("ScriptingStartupScriptEnabled"), _("Startup Script Enabled"), false)
            DEF_BOOL_OPTION(wxT("ScriptingStartupScriptErase"), _("Erase Startup Script Once Run"), false)
            DEF_FILE_OPTION(wxT("ScriptingStartupScriptFilename"), _("Startup Script Filename"), wxEmptyString, wxT("*.*"), wxT("Select startup script"))
            DEF_FILE_OPTION(wxT("ScriptingExportScriptFilename"), _("Export Script Filename"), wxEmptyString, wxT("*.*"), wxT("Select export script"))
        END_OPTION_GROUP
#endif

        DEF_OPTION_GROUP(_("Proxy"))
			DEF_BOOL_OPTION_GROUP_INVERSE(wxT("UseSystemProxySettings"), _("Use System Proxy Settings"), true, boolStoreNormal)
				DEF_STRING_OPTION(wxT("ProxyHost"), _("Proxy Host"), wxEmptyString)
				DEF_INT_OPTION(wxT("ProxyPort"), _("Proxy Port"), 0)
				DEF_STRING_OPTION(wxT("ProxyUsername"), _("Proxy Username"), wxEmptyString)
				DEF_PWD_OPTION(wxT("ProxyPassword"), _("Proxy Password"), wxEmptyString)
			END_BOOL_OPTION_GROUP
		END_OPTION_GROUP

#ifdef LIBROADNAV_DEBUG
		DEF_OPTION_GROUP(_("Debugging"))
			DEF_BOOL_OPTION(wxT("GenerateDebugLog"), _("Generate debug log"), false)
		END_OPTION_GROUP
#endif

		DEF_OPTION_GROUP(_("LCDproc"))
			DEF_OPTION(new LCDprocOption())
		END_OPTION_GROUP

	END_OPTION_GROUP

	OPTIONS_END
    
    // define the dialog, show it
    wxOptionsDialog dlg(parent, rootOptionGroup);
    return dlg.ShowModal();
}

