/*
 *  Roadnav
 *  TTS-Festival.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Text to speech functions using Festival
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _MSC_VER
#pragma warning(disable: 4786)
#endif

#ifdef USE_FESTIVAL

#include <wx/wx.h>
#include <wx/utils.h>
#include <wx/process.h>
#include <wx/txtstrm.h>

#include "libroadnav/Debug.h"
#include "TTS_Festival.h"


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Initialize TTS
///
//////////////////////////////////////////////////////////////////////////////
bool TTS_Festival::InitTTS()
{
	m_initialized = true;
	m_process = new wxProcess(wxPROCESS_REDIRECT);
	m_errcode = wxExecute(wxT("festival --pipe"), wxEXEC_ASYNC, m_process);

	// check to see if it executed
	if (m_errcode == -1 || m_errcode == 0)
		return false;

	// get the command stream (stdin)
	m_cmdstream = m_process->GetOutputStream();
	if (!m_cmdstream)
	{
		if (m_process)
			m_process->Detach();
		m_process = NULL;
		return false;
	}

	m_textstream = new wxTextOutputStream(*m_cmdstream);

	if (!m_textstream)
		return false;

	return true;
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Clean up TTS
///
//////////////////////////////////////////////////////////////////////////////
TTS_Festival::~TTS_Festival()
{
	if (m_initialized && m_process)
	{
		m_process->CloseOutput();
		m_process->Detach();
	}
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Name of this TTS engine
///
//////////////////////////////////////////////////////////////////////////////
wxString TTS_Festival::Name()
{
	return wxT("Festival");
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Retrieve last error message
///
//////////////////////////////////////////////////////////////////////////////
wxString TTS_Festival::GetLastError()
{
	if (!m_initialized)
		return wxT("");

	if (m_errcode == 0)
		return _("Festival exited prematurely");
	if (m_errcode == -1)
		return _("Festival could not be executed!");
	if (!m_process)
		return _("There was an error while executing Festival");
	if (!m_cmdstream)
		return _("The command stream could not be opened for Festival");
	if (!m_textstream)
		return _("The command stream for Festival was opened, but a writer could not be attached");

	return wxT("");
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Retrieve engine status
///
//////////////////////////////////////////////////////////////////////////////
ITTS::ETTSStatus TTS_Festival::GetStatus()
{
	if (!m_initialized)
		return ITTS::TTSStatusOK;

	if (m_errcode == 0 || m_errcode == -1 || !m_process || !m_cmdstream || !m_textstream)
		return ITTS::TTSError;

	return ITTS::TTSStatusOK;
}


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Say some stuff using TTS
///
//////////////////////////////////////////////////////////////////////////////
ITTS::ETTSStatus TTS_Festival::Speak(const wxString & strText)
{   
	// initialize this before starting to speak.. 
	if (!m_initialized && !InitTTS())
		return ITTS::TTSError;

	if (!m_process || !m_cmdstream)
		return ITTS::TTSError;
		
	m_textstream->WriteString(wxT("(SayText \"") + strText + wxT("\")")); 
	
	
	// cleanup on an error
	if (!m_cmdstream->IsOk())
	{
		m_process->CloseOutput();
		m_process->Detach();
		m_process = NULL;
		return ITTS::TTSError;
	}

	return ITTS::TTSStatusOK;
}

#endif // #ifdef USE_FESTIVAL
