/*
 *  Roadnav
 *  TTS_SAPI5.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Text to speech functions using Microsoft SAPI 5. This is threaded, 
/// because it needs to be asynchronous, and for some reason SPF_ASYNC does
/// not work correctly. Its quite odd. 
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _MSC_VER
#pragma warning(disable: 4786)
#endif

#ifdef USE_SAPI5

#include <wx/wx.h>
#include "TTS_SAPI5.h"


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Initialize TTS
///
//////////////////////////////////////////////////////////////////////////////
TTS_SAPI5::TTS_SAPI5() : wxThreadHelper(), m_exitThread(false), m_strLastError(wxT(""))
{
	if (Create() != wxTHREAD_NO_ERROR)
	{
		m_strLastError = wxT("There was an error creating the speech thread!");
		return;
	}

	if (GetThread()->Run() != wxTHREAD_NO_ERROR)
		m_strLastError = wxT("There was an error starting the speech thread!");
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Clean up TTS
///
//////////////////////////////////////////////////////////////////////////////
TTS_SAPI5::~TTS_SAPI5()
{
	m_exitThread = true;
	m_semaphore.Post();
}

void* TTS_SAPI5::Entry()
{
	CComPtr<ISpVoice> m_cpVoice = NULL;
	size_t sz = 0;
	wxString text;

	// initialize COM on this thread
	if (FAILED(CoInitialize(NULL)))
	{
		m_strLastError = wxT("Error initializing COM");
		return NULL;
	}
	
	// create an instance of the speech object
	if (FAILED(m_cpVoice.CoCreateInstance(CLSID_SpVoice)))
	{
		m_strLastError = wxT("Error initializing Microsoft SAPI 5");
		return NULL;
	}

	// wait to speak
	while(m_semaphore.Wait() == wxSEMA_NO_ERROR && !m_exitThread && !GetThread()->TestDestroy())
	{
		m_mutex.Lock();

		sz = m_textToSpeak.Count();

		if (sz > 0)
		{
			text = m_textToSpeak.Item(0);
			m_textToSpeak.RemoveAt(0);
		}
		
		m_mutex.Unlock();

		// ok, do the speaking part now
		if (sz > 0)
			m_cpVoice->Speak(text.wc_str(*wxConvCurrent), 0, 0);
	}

	m_cpVoice.Release();
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Name of this TTS engine
///
//////////////////////////////////////////////////////////////////////////////
wxString TTS_SAPI5::Name()
{
	return wxT("Microsoft SAPI 5");
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Say some stuff using TTS
///
//////////////////////////////////////////////////////////////////////////////
ITTS::ETTSStatus TTS_SAPI5::Speak(const wxString & strText)
{
	if (m_strLastError != wxT(""))
		return ITTS::TTSError;

	m_mutex.Lock();
	m_textToSpeak.Add(strText);
	m_mutex.Unlock();

	m_semaphore.Post();

	return ITTS::TTSStatusOK;
}


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Retrieve last error message
///
//////////////////////////////////////////////////////////////////////////////
wxString TTS_SAPI5::GetLastError()
{
	return m_strLastError;
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Retrieve engine status
///
//////////////////////////////////////////////////////////////////////////////
ITTS::ETTSStatus TTS_SAPI5::GetStatus()
{
	if (m_strLastError != wxT(""))
		return ITTS::TTSError;

	return ITTS::TTSStatusOK;
}

#endif // #ifdef USE_SAPI5
