/*
 *  wxOptionGroup.h
 *
 *  Copyright (c) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Defines option groups to set up a menu hierarchy
///
/// This is part of a generic set of classes designed so that it can be used 
/// with any project, project-specific options should be added 
/// to OptionsLaunch.cpp
///
//////////////////////////////////////////////////////////////////////////////

#ifndef OPTIONGROUP_H
#define OPTIONGROUP_H

#include "wxOptions.h"

// forward declarations needed
class wxOptionGroup;

WX_DECLARE_OBJARRAY(wxOptionGroup, wxOptionGroups);


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implementation for option group
///
/// wxOption groups are used to setup menus that can be traversed in the options
/// dialog. It can either contain a set of option groups (which will be used
///	to define a menu), or a set of options that will be used to define a set 
/// of options to be displayed
///
/// This class is not meant to be a base class. 
///
//////////////////////////////////////////////////////////////////////////////
class wxOptionGroup {
	public:
		
		wxOptionGroup(const wxString& name) : m_name(name), m_parent(NULL) {};

		wxOptionGroup* Add(wxOptionGroup * group);
		wxOptionGroup* Add(wxOption * option);

		bool Save();

		// accessor functions
		wxOptionGroup * GetParent() const { return m_parent; };
		wxString GetName() const { return m_name; };

		bool HasGroups() const { return m_optionGroups.Count() > 0; };
		const wxOptionGroups * GetGroups() const { return &m_optionGroups; };

		bool HasOptions() const { return m_options.Count() > 0; };
		const Options * GetOptions() const { return &m_options; };

		~wxOptionGroup(){
			for (size_t i = 0; i < m_options.Count(); i++)
				delete m_options.Item(i).option;
		};

		void SetParent(wxOptionGroup * parent) { 
			m_parent = parent;
			for (size_t i = 0; i < m_options.Count(); i++)
				m_options.Item(i).option->SetParent(parent);
		};

	private:
		wxOptionGroup(){};

		wxString m_name;
		wxOptionGroup * m_parent;

		wxOptionGroups m_optionGroups;
		Options m_options;

};

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implementation for wxOptionGroupPtr
///
/// Level of indirection to encapsulate an wxOptionGroup because 
/// WX_DECLARE_OBJARRAY and other WX classes/functions want to own everything 
/// you send their way, which causes problems when they delete something and 
/// we still want to reuse whatever it was that we passed it.
///
//////////////////////////////////////////////////////////////////////////////
class wxOptionGroupPtr : public wxObject {
	public:

		DECLARE_DYNAMIC_CLASS(wxOptionGroupPtr);

		wxOptionGroupPtr(wxOptionGroup * group){
			this->group = group;
		};

		wxOptionGroup * group;

	private:
		wxOptionGroupPtr(){}
};

#endif

