/*
 *  wxOptionMacros.h
 *
 *  Copyright (c) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// This defines a bunch of nasty macros that make constructing an option menu
/// much easier/cleaner and more intuitive. However, the underlying framework
/// is still rather nasty. The way this is designed to work is you define a set
/// of option groups, which all contain options. Then you pass them all to
/// an wxOptionsDialog using its static ShowModalDialog method. Example code for 
/// defining the options:
/// 
/// OPTIONS_START(rootOptionGroup)
///     DEF_OPTION_GROUP(_("Group1"))
///     
///         DEF_OPTION_GROUP(_("Inner group1"))
///             DEF_COMBOBOX_ITEMS(g_config, wxT("ConfigValue1"), _("Description"), wxT("DefaultValue"))
///                 DEF_COMBOBOX_ITEM(wxT("DefaultValue"))
///                 DEF_COMBOBOX_ITEM(wxT("Item1"))
///                 DEF_COMBOBOX_ITEM(wxT("Item2"))
///             END_COMBOBOX_ITEMS
///         END_OPTION_GROUP
///         
///         DEF_OPTION_GROUP(_("Inner group2"))
///             DEF_BOOL_OPTION(g_config, wxT("ConfigValue2"), _("Description"), true)
///         END_OPTION_GROUP
///     END_OPTION_GROUP
/// END_OPTIONS
///
/// Then you need to pass the defined option groups (rootOptionGroup) to the 
/// wxOptionsDialog. Obviously, you can use any of the macros defined below to
/// add different options to the groups.
///
/// If you want to define application specific option classes, then refer to
/// Options.h for more information on that. You should inherit from the 
/// base option class (or one of the derived classes), and then define some
/// macros to use it (which is trivial enough, use the below code for info).
///
/// IMPORTANT: You'll notice each option definition defines a configValueName 
/// (2nd param) and a description (3rd param). You should always use wxT() to
/// define the configValueName string, and _() for the description (and other
/// strings too to assist with language support). The reason is that the 
/// configValueName should always be the same, no matter what locale the user
/// is using, just in case they decide to change locales, then they have the 
/// same options. However, the description (ideally) should be localized. 
/// Group names and other strings should probably be localized also. 
///
/// This is part of a generic set of classes designed so that it can be used 
/// with any project, project-specific options should be added 
/// to OptionsLaunch.cpp
///
//////////////////////////////////////////////////////////////////////////////

#ifndef OPTIONMACROS_H
#define OPTIONMACROS_H

#include "wxOptionGroup.h"


// defines a macro to define the root of a group of options
#define OPTIONS_START(root) \
	wxOptionGroup * root = new wxOptionGroup(wxT("Options")); root

#define OPTIONS_END ;

// helper macros 
#define DEF_GROWABLE_THING(ptr) \
	->Add((ptr)
#define END_GROWABLE_THING )

#define DEF_ADD_TO_GROWABLE_THING(ptr) \
	->Add(ptr)

// use these to define groups
#define DEF_OPTION_GROUP(text) \
	DEF_GROWABLE_THING(new wxOptionGroup(text))

#define END_OPTION_GROUP END_GROWABLE_THING

/* 
    nasty macros to make creating option options easier. Generally you
    shouldn't use these unless you're defining new types of options
*/


#define DEF_OPTION(optionPtr) \
	DEF_ADD_TO_GROWABLE_THING(optionPtr)

#define DEF_GROWABLE_OPTION DEF_GROWABLE_THING
#define END_GROWABLE_OPTION END_GROWABLE_THING


/* nasty macros to define options. Use these. */

#define DEF_NULL_OPTION(msg) \
	DEF_OPTION(new nullOption(msg))

#define DEF_BOOL_OPTION(configValueName, description, defaultValue) \
    DEF_OPTION(new wxBoolOption(configValueName, description, defaultValue))

#define DEF_BOOL_OPTION_INVERSE(configValueName, description, defaultValue) \
    DEF_OPTION(new wxBoolOption(configValueName, description, defaultValue, boolStoreInverted))

#define DEF_STRING_OPTION(configValueName, description, defaultValue) \
	DEF_OPTION(new wxStringOption(configValueName, description, defaultValue))

#define DEF_INT_OPTION(configValueName, description, defaultValue) \
	DEF_OPTION(new wxIntOption(configValueName, description, defaultValue))

#define DEF_INT_OPTION_HAS_MINMAX(configValueName, description, defaultValue, min, max) \
	DEF_OPTION(new wxIntOption(configValueName, description, defaultValue, true, min, max))

#define DEF_DOUBLE_OPTION(configValueName, description, defaultValue) \
	DEF_OPTION(new wxDoubleOption(configValueName, description, defaultValue))

#define DEF_DOUBLE_OPTION_HAS_MINMAX(configValueName, description, defaultValue, min, max) \
	DEF_OPTION(new wxDoubleOption(configValueName, description, defaultValue, true, min, max))


#define DEF_SLIDER_INT_OPTION(configValueName, description, defaultValue, min, max) \
	DEF_OPTION(new wxSliderIntOption(configValueName, description, defaultValue, min, max))

#define DEF_SLIDER_DOUBLE_OPTION(configValueName, description, defaultValue, min, max, precision) \
	DEF_OPTION(new wxSliderDoubleOption(configValueName, description, defaultValue, min, max, precision))

#define DEF_PWD_OPTION(configValueName, description, defaultValue) \
	DEF_OPTION(new wxPasswordOption(configValueName, description, defaultValue))

#define DEF_FILE_OPTION(configValueName, description, defaultValue, wildcards, browseMsg) \
	DEF_OPTION(new wxFileOption(configValueName, description, defaultValue, wildcards, browseMsg))

#define DEF_DIRECTORY_OPTION(configValueName, description, defaultValue, browseMsg) \
	DEF_OPTION(new wxDirectoryOption(configValueName, description, defaultValue, browseMsg))

#define DEF_COMBOBOX_ITEMS(configValueName, description, defaultValue) \
	DEF_GROWABLE_THING(new wxComboBoxOption(configValueName, description, defaultValue, false))

#define DEF_COMBOBOX_INDEXED_ITEMS(configValueName, description, defaultValue) \
	DEF_GROWABLE_THING(new wxComboBoxOption(configValueName, description, defaultValue, true))

// DO NOT MIX THE TWO COMBOBOX ITEM TYPES PER ITEM
// it will not probably work the way you expected it to
#define DEF_COMBOBOX_ITEM(item) \
	DEF_ADD_TO_GROWABLE_THING(item)

#define DEF_COMBOBOX_PAIR(index, item) \
	->Add(index, item)

#define END_COMBOBOX_ITEMS END_GROWABLE_THING

/* nasty macros that define options that can have groups... */
#define DEF_BOOL_OPTION_GROUP(configValueName, description, defaultValue, storeInverted) \
    DEF_GROWABLE_OPTION(new wxBoolOption(configValueName, description, defaultValue, storeInverted, true))

#define DEF_BOOL_OPTION_GROUP_INVERSE(configValueName, description, defaultValue, storeInverted) \
	DEF_GROWABLE_OPTION(new wxBoolOption(configValueName, description, defaultValue, storeInverted, false))

#define END_BOOL_OPTION_GROUP END_OPTION_GROUP


#endif
