/////////////////////////////////////////////////////////////////////////////
// Name:        src/common/sizer.cpp
// Purpose:     provide new wxSizer class for layout
// Author:      Robert Roebling and Robin Dunn, contributions by
//              Dirk Holtwick, Ron Lee, Dustin Spicuzza
// Modified by: Ron Lee, Dustin Spicuzza
// Created:
// RCS-ID:      $Id: sizer.cpp 53495 2008-05-08 18:58:04Z VZ $
// Copyright:   (c) Robin Dunn, Robert Roebling
// Licence:     wxWindows licence
//
// This renames the wx implementation of wxOptionSizer and wxOptionSizer
// so that we can use row/col spanning in wx 2.6.x and 2.8.x, since it is
// only present in 2.9.x (well, not yet, but soon)
//
/////////////////////////////////////////////////////////////////////////////

#include "wxOptionSizer.h"

// Uncomment this once http://trac.wxwidgets.org/ticket/9165 is accepted
//#if !wxCHECK_VERSION(2, 9, 0)

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#include <wx/sizer.h>

IMPLEMENT_CLASS(wxOptionSizerFlags, wxObject)
IMPLEMENT_CLASS(wxOptionSizerItem, wxSizerItem)
IMPLEMENT_CLASS(wxOptionSizer, wxSizer)

//---------------------------------------------------------------------------
// wxOptionSizer
//---------------------------------------------------------------------------


int wxOptionSizer::CalcRowsCols()
{
    int nitems = m_children.GetCount();

    if ( nitems )
    {
        // colspan/rowspan affects this, calculate those. Assume they are in the right
        // place, since a later step will catch those errors. 
        wxSizerItemList::compatibility_iterator node = m_children.GetFirst();
        while (node)
        {
            const wxOptionSizerItem * szr = wxDynamicCast(node->GetData(), wxOptionSizerItem);

            if (szr)
            {
                const wxGridSpan span = szr->GetSpan();
                nitems += (span.GetRowspan() * span.GetColspan() - 1);
            }
            
            node = node->GetNext();
        }

        if ( m_user_cols )
        {
            m_cols = m_user_cols;
            m_rows = (nitems + m_user_cols - 1) / m_user_cols;
        }
        else if ( m_user_rows )
        {
            m_cols = (nitems + m_user_rows - 1) / m_user_rows;
            m_rows = m_user_rows;
        }
        else // 0 columns, 0 rows?
        {
            wxFAIL_MSG( _T("grid sizers must have either rows or columns fixed") );

            m_rows = m_cols = 0;
        }

        if (m_rows && m_cols)
            PopulateGrid();
    }

    return nitems;
}

// goes through and creates a grid. Called by CalcRowsCols()
// TODO: Make this cacheable
void wxOptionSizer::PopulateGrid()
{
    wxSizerItemList::const_iterator i = m_children.begin();
    const wxSizerItemList::const_iterator end = m_children.end();

    // initialize the table
    m_gridTable.Clear();
    m_gridTable.Add(NULL, m_rows*m_cols);

    for (int r = 0; r < m_rows; r++)
    {
        int offset = r*m_cols;

        for (int c = 0; c < m_cols; c++)
        {
            // if nothing is already here, then do stuff
            if (!m_gridTable[offset + c])
            {

                // determine if it spans
                wxGridSpan span;
                
                const wxOptionSizerItem * szr = wxDynamicCast(*i, wxOptionSizerItem);
                if (szr)
                    span = szr->GetSpan();
                const int cs = span.GetColspan();
                const int rs = span.GetRowspan();
                
                if (cs > 1 || rs > 1)
                {
                    // since it spans, fill out its places in the array
                    wxASSERT_MSG(c + cs <= m_cols, _T("An item is overflowing its column"));
                    wxASSERT_MSG(r + rs <= m_rows, _T("An item is overflowing its row"));

                    const int maxc = c + cs > m_cols ? m_cols : c + cs;
                    const int maxr = r + rs > m_rows ? m_rows : r + rs;

                    for (int t_r = r; t_r < maxr; t_r++)
                    {
                        const int t_offset = t_r*m_cols;
                        for (int t_c = c; t_c < maxc; t_c++)
                        {
                            wxASSERT_MSG(!m_gridTable[t_offset + t_c], _T("An item is spanning over another item!"));
                            m_gridTable[t_offset + t_c] = (wxSizerItem*)1;
                        }
                    }
                }

                m_gridTable[offset + c] = *i;

                if (++i == end)
                    break;
            }   
        }

        if (i == end)
            break;
    }
}

// begins iteration
wxSizerItem* wxOptionSizer::BeginIterateGrid(int *row, int *col, wxGridSpan * span)
{
    m_grid_cur_col = 0;
    m_grid_cur_row = 0;

    return IterateGrid(row, col, span);
}

// returns row, column, span of each cell in succession, properly skipping
// spanned cells. Useful for any classes that may derive from this
wxSizerItem* wxOptionSizer::IterateGrid(int *row, int *col, wxGridSpan * span)
{
    wxSizerItem *ret_item = NULL;

    while (m_grid_cur_row < m_rows)
    {
        int offset = m_grid_cur_row*m_cols;

        while (m_grid_cur_col< m_cols)
        {
            // should we skip this item?
            wxSizerItem* item = m_gridTable[offset + m_grid_cur_col];
            if (item && item != (wxSizerItem*)1)
            {
                // nope, found an item... 
                ret_item = item;
                break;
            }

            ++m_grid_cur_col;
        }
        
        // break twice
        if (ret_item)
            break;

        m_grid_cur_col = 0;
        ++m_grid_cur_row;
    }

    // return the information that the caller requested
    if (ret_item)
    {
        if (row) *row = m_grid_cur_row;
        if (col) *col = m_grid_cur_col;
        if (span)
        {
            const wxOptionSizerItem * szr = wxDynamicCast(ret_item, wxOptionSizerItem);
            if (szr)
                *span = szr->GetSpan();
            else
                *span = wxDefaultSpan;
        }
    }

    // no infinite loops
    ++m_grid_cur_col;

    return ret_item;
}


// calculates the edges of the grid and saves them. Only called from
// RecalcSizes(). Ideally, this should work correctly for all derived
// classes, and doesn't need to be reimplemented
void wxOptionSizer::CalcGridPoints()
{
    int i; 
    wxPoint pt( GetPosition() );
    
    m_colPts.Clear();
    m_colPts.Add(0,m_cols);

    m_colPts[0] = pt.x;
   
    for (i = 1; i < m_cols; i++)
        // if negative, then assume zero width
        if (m_colWidths[i] < 0)
            m_colPts[i] = m_colPts[i-1];
        else
            m_colPts[i] = m_colPts[i-1] + m_colWidths[i-1] + m_hgap;
    
    m_rowPts.Clear();
    m_rowPts.Add(0,m_rows);

    m_rowPts[0] = pt.y;
   
    for (i = 1; i < m_rows; i++)
        // if negative, then assume zero height
        if (m_rowHeights[i] < 0)
            m_rowPts[i] = m_rowPts[i-1];
        else
            m_rowPts[i] = m_rowPts[i-1] + m_rowHeights[i-1] + m_vgap;
}


// the purpose of this function is to calculate the size of each child, 
// and then actually set it
void wxOptionSizer::RecalcSizes()
{
    if ( CalcRowsCols() == 0 )
        return;

    const wxSize sz( GetSize() );

    int row, col, i;
    wxGridSpan span;
    wxSizerItem* item;
    
    // next, determine widths/heights
    CalcGridWidthsAndHeights();

    // and then the edges
    CalcGridPoints();

    
    for (item = BeginIterateGrid(&row, &col, &span); 
         item; 
         item = IterateGrid(&row, &col, &span))
    {
        if (m_rowHeights[row] == -1 || m_colWidths[col] == -1)
            continue;
    
        const int cs = span.GetColspan();
        const int rs = span.GetRowspan();
        int w = m_colWidths[col];
        int h = m_rowHeights[row];

        if (cs > 1 || rs > 1)
        {   
            // limit the colspan/rowspan so it doesn't overflow
            const int maxrs = row + rs > m_rows ? m_rows : row + rs;
            const int maxcs = col + cs > m_cols ? m_cols : col + cs;

            for (i = row+1; i < maxrs; i++)
                if (m_rowHeights[i] > 0)
                    h += m_vgap + m_rowHeights[i];

            for (i = col+1; i < maxcs; i++)
                if (m_colWidths[i] > 0)
                    w += m_hgap + m_colWidths[i];
        }

        SetItemBounds( item, m_colPts[col], m_rowPts[row], w, h);
    }
}

// helper function to decide how wide an individual cell should
// be when dealing with a rowspan/colspan item
static inline
int DoCalcAvgSpan(const int initsz, const int span, const int gap)
{
    int sz = initsz;

    // adjust for the gaps
    sz = sz - (span-1)*gap;
    if (sz < 0)
        sz = 0;

    // then divide by the span so it doesn't effect overall width
    if (sz % span)
        sz = (sz + span)/span;
    else
        sz = sz / span;

    return sz;
}

// helper function -- children shouldn't need to override this. Called 
// from RecalcSizes() only
void wxOptionSizer::SetItemBounds( wxSizerItem *item, int x, int y, int w, int h )
{
    wxPoint pt( x,y );
    wxSize sz( item->GetMinSizeWithBorder() );
    int flag = item->GetFlag();

    if ((flag & wxEXPAND) || (flag & wxSHAPED))
    {
       sz = wxSize(w, h);
    }
    else
    {
        if (flag & wxALIGN_CENTER_HORIZONTAL)
        {
            pt.x = x + (w - sz.x) / 2;
        }
        else if (flag & wxALIGN_RIGHT)
        {
            pt.x = x + (w - sz.x);
        }

        if (flag & wxALIGN_CENTER_VERTICAL)
        {
            pt.y = y + (h - sz.y) / 2;
        }
        else if (flag & wxALIGN_BOTTOM)
        {
            pt.y = y + (h - sz.y);
        }
    }

    item->SetDimension(pt, sz);
}



//---------------------------------------------------------------------------
// wxOptionSizer
//---------------------------------------------------------------------------

wxOptionSizer::wxOptionSizer( int rows, int cols, int vgap, int hgap )
                : m_user_rows( ( cols == 0 && rows == 0 ) ? 1 : rows )
                , m_user_cols( cols )
                , m_vgap( vgap )
                , m_hgap( hgap )
                ,
                 m_flexDirection(wxBOTH),
                 m_growMode(wxFLEX_GROWMODE_SPECIFIED)
{
}

wxOptionSizer::wxOptionSizer( int cols, int vgap, int hgap )
                : m_user_rows( cols == 0 ? 1 : 0 )
                , m_user_cols( cols )
                , m_vgap( vgap )
                , m_hgap( hgap )
                ,
                 m_flexDirection(wxBOTH),
                 m_growMode(wxFLEX_GROWMODE_SPECIFIED)
{
}

wxOptionSizer::~wxOptionSizer()
{
}


// helper function.. adjusts either the width or height array
static inline
void DoAdjustWHArrayForItem(const int sz, 
                            const int start_idx, 
                            const int span_len,
                            wxArrayInt &array, 
                            const int arraysz,
                            const int gap )
{
    int i, tot_sz = 0, tot_empty = 0, tot_full = 0;

    // sum the number of items spanned
    int limit = start_idx + span_len < arraysz ? start_idx + span_len : arraysz;
    
    // get the size that exists already
    for (i = start_idx; i < limit; i++)
        if (array[i] == 0)
            tot_empty += 1;
        else if (array[i] > 0)
        {
            tot_sz += gap + array[i];
            tot_full += 1;
        }
        
    // if item is bigger than the total span
    if (sz > tot_sz)
    {
        // get the average remaining size and distribute it
        // either across anything that is empty, or failing
        // that, across anything that is full
        int div;
        int szleft = sz - tot_sz;

        if (tot_empty)
        {
            div = DoCalcAvgSpan(szleft, tot_empty, gap);

            // evenly across all empty items
            for (i = start_idx; tot_empty && i < limit; i++)
                if (array[i] == 0)
                {
                    tot_empty -= 1;
                    array[i] = div;
                }
        } else 
        {
            div = DoCalcAvgSpan(szleft, tot_full, gap);

            // evenly across all non-hidden items
            for (i = start_idx; i < limit; i++)
                if (div > array[i] && array[i] > 0)
                    array[i] = div;
        }
    }
}

// this routine assigns the rowHeight/colWidth array sizes
void wxOptionSizer::CalcMaxRowColSz(bool calcMin)
{
    // We have to recalculate the sizes in case the item minimum size has
    // changed since the previous layout, or the item has been hidden using
    // wxSizer::Show(). If all the items in a row/column are hidden, the final
    // dimension of the row/column will be -1, indicating that the column
    // itself is hidden.
    //
    // If a column is zero, then it is (potentially) available for spanning
    m_rowHeights.assign(m_rows, 0); 
    m_colWidths.assign(m_cols, 0);

    int row, col, i;
    wxGridSpan span;
    wxSizerItem * item;

    // two passes: non-spanning items first, then any spannable
    // items the second time around. This allows us to minimize
    // the amount of overflow present (to some extent)
    //
    // this is *almost* as good as AdjustForOverflow() is. If we could 
    // get that working, it would probably be slightly better. However, 
    // given the current grid mechanism its pretty difficult (and slow) 
    // to implement it -- IMHO this works for most cases, so its
    // not worth the tradeoff. Refer to the complex flexgridsizer
    // portion of the Layout sample for examples of layouts that are 
    // broken with this.

    for ( item = BeginIterateGrid(&row, &col, &span);
          item;
          item = IterateGrid(&row, &col, &span))
    {

        const int rs = span.GetRowspan();
		const int cs = span.GetColspan();

        if ( !item->IsShown())
        {
            // initialize hidden stuff ONLY if it hasn't 
            // already been initialized
            for (i = row; i < row+rs && i < m_rows; i++)
                if (m_rowHeights[i] == 0)
                    m_rowHeights[i] = -1;
            for (i = col; i < col+cs && i < m_cols; i++)
                if (m_colWidths[i] == 0)
                    m_colWidths[i] = -1;
        }
        else
        {
            wxSize sz;

            // skip purely spanning items
            if (rs > 1 && cs > 1)
                continue;

            if (calcMin)
                sz = item->CalcMin();
            else
                sz = item->GetMinSizeWithBorder();

            if (rs <= 1 && sz.y > m_rowHeights[row] )
                m_rowHeights[row] = sz.y;

            if (cs <= 1 && sz.x > m_colWidths[col])
                m_colWidths[col] = sz.x;
        }
    }

    // do it again, span items only
    for ( item = BeginIterateGrid(&row, &col, &span);
          item;
          item = IterateGrid(&row, &col, &span))
    {
        if ( item->IsShown())
        {
            wxSize sz;

            const int rs = span.GetRowspan() < 1 ? 1 : span.GetRowspan();
		    const int cs = span.GetColspan() < 1 ? 1 : span.GetColspan();

            // skip non-spanning items
            if (rs <= 1 && cs <= 1)
                continue;

            if (calcMin)
                sz = item->CalcMin();
            else
                sz = item->GetMinSizeWithBorder();

            DoAdjustWHArrayForItem(sz.x, col, cs, m_colWidths, m_cols, m_vgap );   
            DoAdjustWHArrayForItem(sz.y, row, rs, m_rowHeights, m_rows, m_hgap );    
        }
    }
}


// helper of CalcGridWidthsAndHeights() which is called for rows/columns 
// separately. This grows out the rows/columns
//
// parameters:
//      delta: the extra space, we do nothing unless it's positive
//      growable: indices or growable rows/cols in sizes array
//      sizes: the height/widths of rows/cols to adjust
//      proportions: proportions of the growable rows/cols or NULL if they all
//                   should be assumed to have proportion of 1
static void
DoAdjustForGrowables(const int delta,
                     const wxArrayInt& growable,
                     wxArrayInt& sizes,
                     const wxArrayInt *proportions)
{
    if ( delta <= 0 )
        return;

    // total sum of proportions of all non-hidden rows
    int sum_proportions = 0;

    // number of currently shown growable rows
    int num = 0;

    const int max_idx = sizes.size();

    // this parameter is used in both for loops, either refers to
    // growable[] or sizes[], depending on the value of proportions
    size_t count = growable.size();
    size_t idx;

    // if this is NULL, then we automatically resize all items
    if (!proportions)
        num = count = max_idx;

    else 
    {
        for ( idx = 0; idx < count; idx++ )
        {
            // Since the number of rows/columns can change as items are
            // inserted/deleted, we need to verify at runtime that the
            // requested growable rows/columns are still valid.
            if ( growable[idx] >= max_idx )
                continue;

            // If all items in a row/column are hidden, that row/column will
            // have a dimension of -1.  This causes the row/column to be
            // hidden completely.
            if ( sizes[growable[idx]] == -1 )
                continue;

            sum_proportions += (*proportions)[idx];
            num++;
        }

        if ( !num )
            return;
    }

    // the remaining extra free space, adjusted during each iteration
    for ( idx = 0; idx < count; idx++ )
    {
        // skip this check if its null
        if (proportions && growable[idx] >= max_idx )
            continue;

        int real_idx = proportions ? growable[idx] : idx;

        if ( sizes[real_idx] == -1 )
            continue;

        int cur_delta;
        if ( sum_proportions == 0 )
            // no growable rows/cols -- divide extra space evenly among all
            cur_delta = delta/num;
        
        else // allocate extra space proportionally
        {
            const int cur_prop = (*proportions)[idx];
            cur_delta = (delta*cur_prop)/sum_proportions;
        }

        sizes[real_idx] += cur_delta;
    }
}


// this is only called from RecalcSizes()
void wxOptionSizer::CalcGridWidthsAndHeights()
{
    const wxSize sz( GetSize() );

    CalcMaxRowColSz(false);

    // since this is the flex sizer, we need to adjust the elements to grow
    // correctly and flex accordingly

    // wxFLEX_GROWMODE_ALL is handled specially since both of the following if 
    // statements will pass it through. A NULL as the fourth parameter
    // to DoAdjustForGrowables means to distribute the extra weight across all
    // rows -- which should only be done if wxFLEX_GROWMODE_ALL is specified 
    // AND it is a non-flexible direction

    // horizontal direction
    if ( (m_flexDirection & wxHORIZONTAL) || (m_growMode != wxFLEX_GROWMODE_NONE) )
    {
        DoAdjustForGrowables
        (
            sz.x - m_calculatedMinSize.x,
            m_growableCols,
            m_colWidths,
            m_flexDirection & wxHORIZONTAL || m_growMode == wxFLEX_GROWMODE_SPECIFIED
                ? &m_growableColsProportions : NULL
        );
    }

    // then for the vertical direction
    if ( (m_flexDirection & wxVERTICAL) || (m_growMode != wxFLEX_GROWMODE_NONE) )
    {
        // pass NULL instead of proportions if the grow mode is ALL as we
        // should treat all rows as having proportion of 1 then
        DoAdjustForGrowables
        (
            sz.y - m_calculatedMinSize.y,
            m_growableRows,
            m_rowHeights,
            m_flexDirection & wxVERTICAL || m_growMode == wxFLEX_GROWMODE_SPECIFIED 
                ? &m_growableRowsProportions : NULL
        );
    }
}


// helper function used in CalcMin() to sum up the sizes of non-hidden items
static inline
int SumArraySizes(const wxArrayInt& sizes, int gap)
{
    // Sum total minimum size, including gaps between rows/columns.
    // -1 is used as a magic number meaning empty row/column.
    int total = 0;

    const size_t count = sizes.size();
    for ( size_t n = 0; n < count; n++ )
    {
        if ( sizes[n] != -1 )
        {
            if ( total )
                total += gap; // separate from the previous column

            total += sizes[n];
        }
    }

    return total;
}


wxSize wxOptionSizer::CalcMin()
{
    // Number of rows/columns can change as items are added or removed.
    if ( !CalcRowsCols() )
        return wxSize();

    // find min sizes
    CalcMaxRowColSz(true);

    // the logic in CalcMin works when we resize flexibly in both directions
    // but maybe this is not the case
    if ( m_flexDirection != wxBOTH )
    {
        // select the array corresponding to the direction in which we do *not*
        // resize flexibly
        wxArrayInt& array = m_flexDirection == wxVERTICAL ? m_colWidths
                                                          : m_rowHeights;

        const size_t count = array.GetCount();

        // find the largest value in this array
        size_t n;
        int largest = 0;

        for ( n = 0; n < count; ++n )
        {
            if ( array[n] > largest )
                largest = array[n];
        }

        // and now fill it with the largest value
        for ( n = 0; n < count; ++n )
        {
            // don't touch hidden rows
            if ( array[n] != -1 )
                array[n] = largest;
        }
    }

    // sum arrays to return the final size
    m_calculatedMinSize = wxSize(SumArraySizes(m_colWidths, m_hgap), SumArraySizes(m_rowHeights, m_vgap));

    return m_calculatedMinSize;
}


void wxOptionSizer::AddGrowableRow( size_t idx, int proportion )
{
    m_growableRows.Add( idx );
    m_growableRowsProportions.Add( proportion );
}

void wxOptionSizer::AddGrowableCol( size_t idx, int proportion )
{
    m_growableCols.Add( idx );
    m_growableColsProportions.Add( proportion );
}

// helper function for RemoveGrowableCol/Row()
static void
DoRemoveFromArrays(size_t idx, wxArrayInt& items, wxArrayInt& proportions)
{
    const size_t count = items.size();
    for ( size_t n = 0; n < count; n++ )
    {
        if ( (size_t)items[n] == idx )
        {
            items.RemoveAt(n);
            proportions.RemoveAt(n);
            return;
        }
    }

    wxFAIL_MSG( _T("column/row is already not growable") );
}

void wxOptionSizer::RemoveGrowableCol( size_t idx )
{
    DoRemoveFromArrays(idx, m_growableCols, m_growableColsProportions);
}

void wxOptionSizer::RemoveGrowableRow( size_t idx )
{
    DoRemoveFromArrays(idx, m_growableRows, m_growableRowsProportions);
}

//#endif
