/////////////////////////////////////////////////////////////////////////////
// Name:        Taken from wx/sizer.h
// Purpose:     provide wxSizer class for layout
// Author:      Robert Roebling and Robin Dunn
// Modified by: Ron Lee, Vadim Zeitlin (wxSizerFlags), Dustin Spicuzza
// Created:
// RCS-ID:      $Id: sizer.h 53495 2008-05-08 18:58:04Z VZ $
// Copyright:   (c) Robin Dunn, Robert Roebling
// Licence:     wxWindows licence
//
// This renames the wx implementation of wxGridSizer and wxOptionSizer
// so that we can use row/col spanning in wx 2.6.x and 2.8.x, since it is
// only present in 2.9.x (well, not yet, but soon)
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __WX_OPTION_SIZER_H
#define __WX_OPTION_SIZER_H

#include <wx/wxprec.h>
#include <wx/version.h>

#include <wx/sizer.h>
#include <wx/gbsizer.h>

// Uncomment this once http://trac.wxwidgets.org/ticket/9165 is accepted
//#if wxCHECK_VERSION(2, 9, 0)

    //typedef wxFlexGridSizer wxOptionSizer;
    //typedef wxSizerFlags wxOptionSizerFlags;

//#else


typedef wxGBPosition wxGridPosition;
typedef wxGBSpan wxGridSpan;


// ----------------------------------------------------------------------------
// wxSizerFlags: flags used for an item in the sizer
// ----------------------------------------------------------------------------

class wxOptionSizerFlags : public wxSizerFlags, public wxObject
{
public:
    // construct the flags object initialized with the given proportion (0 by
    // default)
    wxOptionSizerFlags(int proportion = 0) : wxSizerFlags(proportion)
    {}

    // setters for all sizer flags, they all return the object itself so that
    // calls to them can be chained

    // wxGridSizer derived classes only
    wxOptionSizerFlags& ColSpan(int cols) 
    {
        m_span.SetColspan(cols);
        return *this;
    }

    wxOptionSizerFlags& RowSpan(int rows)
    {
        m_span.SetRowspan(rows);
        return *this;
    }

    wxOptionSizerFlags& Span(wxGridSpan span)
    {
        m_span = span;
        return *this;
    }
    
    // accessors for wxSizer only
    wxGridSpan GetSpan() const { return m_span; }


private:
    wxGridSpan m_span;
    DECLARE_CLASS(wxOptionSizerFlags)
};


// ----------------------------------------------------------------------------
// wxOptionSizerItem
// ----------------------------------------------------------------------------

class wxOptionSizerItem : public wxSizerItem
{
public:
    // window with flags
    wxOptionSizerItem(wxWindow *window, const wxOptionSizerFlags& flags) :
        wxSizerItem(window, flags)
    {
        m_span = flags.GetSpan();
    }

    // sizer with flags
    wxOptionSizerItem(wxSizer *sizer, const wxOptionSizerFlags& flags) :
        wxSizerItem(sizer, flags)
    {
        m_span = flags.GetSpan();
    }

    // spacer with flags
    wxOptionSizerItem(int width, int height, const wxOptionSizerFlags& flags) :
        wxSizerItem(width, height, flags)
    {
        m_span = flags.GetSpan();
    }

    // virtual for compatibility with wxGBSizer.. the return value is actually
    // meaningless
    virtual bool SetSpan(const wxGridSpan& span)
        { m_span = span; return true; }
    wxGridSpan GetSpan() const
        { return m_span; }

protected:

    // wxGridSizer specific
    wxGridSpan  m_span;

private:
    DECLARE_CLASS(wxOptionSizerItem)
    DECLARE_NO_COPY_CLASS(wxOptionSizerItem)
};

WX_DEFINE_EXPORTED_ARRAY_PTR( wxSizerItem*, wxSizerItemPtrArray );


//---------------------------------------------------------------------------
// wxOptionSizer
//---------------------------------------------------------------------------

/*
    This is the wxOptionSizer from patch #9165, which
    is a rework of the existing wxGridSizer/wxFlexGridSizer
    classes. We use these instead for backwards compatibility.
*/

class wxOptionSizer: public wxSizer
{
public:
    // ctors/dtor
    wxOptionSizer( int rows, int cols, int vgap, int hgap );
    wxOptionSizer( int cols, int vgap = 0, int hgap = 0 );
    virtual ~wxOptionSizer();

    // set the rows/columns which will grow (the others will remain of the
    // constant initial size)
    void AddGrowableRow( size_t idx, int proportion = 0 );
    void RemoveGrowableRow( size_t idx );
    void AddGrowableCol( size_t idx, int proportion = 0 );
    void RemoveGrowableCol( size_t idx );


    // the sizer cells may grow in both directions, not grow at all or only
    // grow in one direction but not the other

    // the direction may be wxVERTICAL, wxHORIZONTAL or wxBOTH (default)
    void SetFlexibleDirection(int direction) { m_flexDirection = direction; }
    int GetFlexibleDirection() const { return m_flexDirection; }

    // note that the grow mode only applies to the direction which is not
    // flexible
    void SetNonFlexibleGrowMode(wxFlexSizerGrowMode mode) { m_growMode = mode; }
    wxFlexSizerGrowMode GetNonFlexibleGrowMode() const { return m_growMode; }

    // Read-only access to the row heights and col widths arrays
    const wxArrayInt& GetRowHeights() const { return m_rowHeights; }
    const wxArrayInt& GetColWidths() const  { return m_colWidths; }

    // implementaion details
    virtual wxSize CalcMin();
    virtual void RecalcSizes();

protected:

  // these two are only used by CalcRowsCols() to determine the real
    // number of rows and/or columns
    int    m_user_rows, m_user_cols;

    // these are used by everything else
    int    m_rows, m_cols;
    int    m_vgap;
    int    m_hgap;

    // return the number of total items and the number of columns and rows

    void SetItemBounds( wxSizerItem *item, int x, int y, int w, int h );

    // helper functions to handle the grid correctly: call them
    // in the order listed only!

    // figures out how many rows and columns there are, sets m_rows/m_cols
    virtual int CalcRowsCols();
    // fills in the grid -- called by CalcRowsCols
    virtual void PopulateGrid();
    // fills in m_rowHeights and m_colWidths
    virtual void CalcGridWidthsAndHeights();
    // fills in m_rowPts and m_colPts
    void CalcGridPoints();

    // iterates over the grid and returns useful information
    wxSizerItem* BeginIterateGrid(int *row, int *col, wxGridSpan *span);
    wxSizerItem* IterateGrid(int *row, int *col, wxGridSpan *span);

    // the gridtable is calculated by PopulateGrid(). This is an array that
    // is m_cols*m_rows long, denoting each cell. It contains either a pointer
    // to that item, a NULL (empty cell), or a 1 (last parts of spanning item)
    wxSizerItemPtrArray m_gridTable;

    // keeps track of stuff for the IterateGrid related functions
    int m_grid_cur_col,
        m_grid_cur_row;
    
    // points rows/columns located at
    wxArrayInt m_rowPts,
               m_colPts;

    // the heights/widths of all rows/columns
    wxArrayInt  m_rowHeights,
                m_colWidths;

    void CalcMaxRowColSz(bool calcMin);

    // indices of the growable columns and rows
    wxArrayInt  m_growableRows,
                m_growableCols;

    // proportion values of the corresponding growable rows and columns
    wxArrayInt  m_growableRowsProportions,
                m_growableColsProportions;

    // parameters describing whether the growable cells should be resized in
    // both directions or only one
    int m_flexDirection;
    wxFlexSizerGrowMode m_growMode;

    // saves CalcMin result to optimize RecalcSizes
    wxSize m_calculatedMinSize;

private:
    DECLARE_CLASS(wxOptionSizer)
    DECLARE_NO_COPY_CLASS(wxOptionSizer)
};


//#endif

#endif
