/*
 *  Options.cpp
 *
 *  Copyright (c) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Defines implementation of option-related classes to be used to create 
/// menus and such. 
///
/// This is part of a generic set of classes designed so that it can be used 
/// with any project, project-specific options should be added 
/// to OptionsLaunch.cpp
///
//////////////////////////////////////////////////////////////////////////////

#include <wx/wx.h>
#include <wx/event.h>

#include "wxOptions.h"
#include <wx/arrimpl.cpp>

// definitions 
WX_DEFINE_OBJARRAY(Options);
IMPLEMENT_DYNAMIC_CLASS(wxOptionPtr, wxObject);



/* option implementation stuff below */


wxBoolOption * wxBoolOption::Add(wxOption * option) 
{
    m_options.Add(option);
    return this;
};

void wxBoolOption::DoDisplay(wxOptionSizer * sizer, wxWindow * parent) 
{
	m_checkBox = new wxCheckBox(parent, wxID_ANY, wxT(""));
	m_checkBox->SetValue(m_defaultValue);

	OPTION_ADD_DEFAULT_LABEL;
	sizer->Add(m_checkBox);
    
	if (m_options.Count()){

		// need to pass a pointer to *this, since the function doesn't get called
		// with the right pointer apparently... 
		m_checkBox->Connect(m_checkBox->GetId(), wxEVT_COMMAND_CHECKBOX_CLICKED, 
			wxCommandEventHandler(wxBoolOption::OnCheck), 
			wxDynamicCast(new wxOptionPtr(this),wxObject));

		// if we have children, then display each one of those options and
		// enable them accordingly
		for (size_t i = 0; i < m_options.Count(); i++){
			m_options.Item(i).option->Display(sizer, parent);
			m_options.Item(i).option->Enable(ChildEnabled(true));
		}
	}
}

void wxBoolOption::OnCheck(wxCommandEvent& event)
{
	// this Nasty(TM) hack magic is required because of this fn is called with an invalid 
	// *this pointer, most likely because of multiple inheritance
	wxBoolOption * _this = (wxBoolOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;

	for (size_t i = 0; i < _this->m_options.Count(); i++)
		_this->m_options.Item(i).option->Enable(_this->ChildEnabled(true));
}

void wxBoolOption::OnWindowDestroy() 
{ 
    m_defaultValue = m_checkBox->IsChecked(); 
    
	// call on any children
    for (size_t i = 0; i < m_options.Count(); i++)
        m_options.Item(i).option->OnWindowDestroy();
}

bool wxBoolOption::DoSave() 
{ 
	bool ret = wxConfigBase::Get()->Write(m_configValueName, (long)(m_storeInverted == boolStoreInverted ? !m_defaultValue : m_defaultValue));
    
	// call on any children
    for (size_t i = 0; i < m_options.Count(); i++)
        ret = ret && m_options.Item(i).option->Save();
        
    return ret;
}

bool wxBoolOption::Enable(bool enable) 
{ 
    bool ret = m_checkBox->Enable(enable); 

	// call on any children
    for (size_t i = 0; i < m_options.Count(); i++)
        m_options.Item(i).option->Enable(ChildEnabled(enable));
    
    return ret;
}

void wxBoolOption::SetParent(wxOptionGroup * parent)
{
    m_parentGroup = parent;

	// call on any children
    for (size_t i = 0; i < m_options.Count(); i++)
        m_options.Item(i).option->SetParent(parent);
}

wxBoolOption::~wxBoolOption()
{
	// delete kids
    for (size_t i = 0; i < m_options.Count(); i++)
        delete m_options.Item(i).option;
}

// this is a helper function used in the code above used to
// determine whether the children should be enabled or not
bool wxBoolOption::ChildEnabled(bool enable)
{
    if (m_enableIfChecked)
        return enable && m_checkBox->IsChecked();
    else
        return enable && !m_checkBox->IsChecked();
}




void wxFileOption::DoDisplay(wxOptionSizer * sizer, wxWindow * parent) 
{
	OPTION_ADD_DEFAULT_LABEL;
	wxBoxSizer * boxSizer = new wxBoxSizer(wxHORIZONTAL);
	boxSizer->Add(m_textCtrl = new wxTextCtrl(parent, wxID_ANY, m_defaultValue), 1);
	boxSizer->Add(m_browseButton = new wxButton(parent, wxID_ANY, _("Browse")));
	
	sizer->Add(boxSizer);

	// pass a pointer to the this pointer... see wxBoolOption::DoDisplay
	m_browseButton->Connect(m_browseButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
		wxCommandEventHandler(wxFileOption::OnBrowse), wxDynamicCast(new wxOptionPtr(this),wxObject));
}

void wxFileOption::OnBrowse(wxCommandEvent& event)
{
	// see above this this Nasty(TM) hack
	wxFileOption * _this = (wxFileOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;

	wxFileDialog dlg(_this->m_browseButton->GetParent(), _this->m_browseMsg, wxT(""), _this->m_textCtrl->GetValue(), _this->m_wildcards, 
#if wxCHECK_VERSION(2,8,0)
		wxFD_OPEN
#else
		wxOPEN
#endif
	);

	if (dlg.ShowModal() == wxID_OK)
		_this->m_textCtrl->SetValue(dlg.GetPath());
}

void wxDirectoryOption::DoDisplay(wxOptionSizer * sizer, wxWindow * parent) 
{
	OPTION_ADD_DEFAULT_LABEL;
	wxBoxSizer * boxSizer = new wxBoxSizer(wxHORIZONTAL);
	boxSizer->Add(m_textCtrl = new wxTextCtrl(parent, wxID_ANY, m_defaultValue), 1);
	boxSizer->Add(m_browseButton = new wxButton(parent, wxID_ANY, _("Browse")));
	
	sizer->Add(boxSizer);

	// same as above
	m_browseButton->Connect(m_browseButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
		wxCommandEventHandler(wxDirectoryOption::OnBrowse), wxDynamicCast(new wxOptionPtr(this), wxObject));
}

void wxDirectoryOption::OnBrowse(wxCommandEvent& event)
{
	// once again, nothing new here
	wxDirectoryOption * _this = (wxDirectoryOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;

	wxDirDialog dlg(_this->m_browseButton->GetParent(), _this->m_browseMsg, _this->m_textCtrl->GetValue());

	if (dlg.ShowModal() == wxID_OK)
		_this->m_textCtrl->SetValue(dlg.GetPath());
}

// this version of Add() and the next version should NOT be mixed together, use
// one or the other!
wxComboBoxOption* wxComboBoxOption::Add(const wxString& item)
{
	m_items.Add(item);
	m_indexes.Add(m_indexes.Count());

	if (m_comboBox)
		m_comboBox->Append(item);

	return this;
}

wxComboBoxOption* wxComboBoxOption::Add(long index, const wxString& item)
{
	m_items.Add(item);
	m_indexes.Add(index);

	if (m_comboBox)
		m_comboBox->Append(item);

	return this;
}


void wxComboBoxOption::DoDisplay(wxOptionSizer * sizer, wxWindow * parent)
{
	size_t i;

	// decide the default value if we haven't already
	if (!m_hasDefault)
	{
		if (!m_saveIndex)
			m_defaultValue = wxConfigBase::Get()->Read(m_configValueName, m_defaultValue); 
		else 
		{
			// if we're doing things by index, then its a lot harder.. 
			long def = 0;
			size_t def_i = 0;
			bool found = false;

			// search the items for the default index
			for (i = 0; i < m_items.Count(); i++)
				if (m_items[i] == m_defaultValue){
					def = m_indexes[i];
					def_i = i;
					break;
				}

			// get the user-defined index
			long idx = wxConfigBase::Get()->Read(m_configValueName, def );	

			// then search the indexes for the default item, and set it
			for (i = 0; i < m_indexes.Count(); i++)
				if (m_indexes[i] == idx){
					m_defaultValue = m_items[i];
					found = true;
					break;
				}

			if (!found)
				m_defaultValue = m_items[def_i];

		}

		m_hasDefault = true;
	}

	OPTION_ADD_DEFAULT_LABEL;
	sizer->Add(m_comboBox = new wxComboBox(parent, wxID_ANY, m_defaultValue, wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_READONLY), 1);
	
	m_comboBox->Append(m_items);

	// Set the selection... but fall back to the original default value 
	// if the passed in default value isn't found, unless that doesn't work, then
	// just select the first one
	if (!m_comboBox->SetStringSelection(m_defaultValue))
		if (!m_comboBox->SetStringSelection(m_originalDefaultValue))
			m_comboBox->SetSelection(0);
}

bool wxComboBoxOption::DoSave()
{ 
	// this is the easy case: just save the string
	if (!m_saveIndex)
		return wxConfigBase::Get()->Write(m_configValueName, m_defaultValue); 

	// otherwise search for the right item, and save its index
	for ( size_t i = 0; i < m_items.Count(); i++ )
		if ( m_items[i] == m_defaultValue )
			return wxConfigBase::Get()->Write(m_configValueName, m_indexes[i]);

	// we should never actually get here
	wxASSERT_MSG(0,wxT("An invalid item was somehow selected"));
	return false;
}
        
