/*
 *  wxOptions.h
 *
 *  Copyright (c) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Defines option-related classes to be used to create menus and such. There
/// is a set of macros defined to make using these options easier (see 
/// OptionMacros.h for those)
///
/// This is part of a generic set of classes designed so that it can be used 
/// with any project, project-specific options should be added 
/// to OptionsLaunch.cpp
///
//////////////////////////////////////////////////////////////////////////////
 

#ifndef OPTIONS_H
#define OPTIONS_H

#include <wx/wx.h>
#include <wx/confbase.h>
#include <wx/gbsizer.h>
#include <wx/object.h>
#include <wx/dynarray.h>

#include "wxOptionSizer.h"

// forward declarations needed
class wxOptionPtr;
class wxOptionGroup;

WX_DECLARE_OBJARRAY(wxOptionPtr, Options);


// useful macro
#define OPTION_ADD_DEFAULT_LABEL \
	sizer->Add(new wxStaticText(parent, wxID_ANY, m_description), 0, wxALIGN_CENTER_VERTICAL)


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implementation for option
///
/// Abstract base class that all options derive from. Useful so we can store 
/// lists of wxOption* and call the common interface without having to know
/// what type it is. 
///
/// Any option that supports multiple objects contained inside of it has an
/// Add() function that returns a pointer to itself so that you can chain 
/// them together and add multiple options. Combined with macros, this makes
/// creating the menus significantly easier, and far more flexible since the
/// behavior of all options can be changed at the same time, ensuring a
/// consistant look/feel.
///
/// If you need to get events from a control that you added, then your class
/// must do multiple inheritance and inherit from wxOption and wxEvtHandler.
/// Then you can use Connect() to dynamically create an event handler for the
/// control. Be warned: because of the inheritance, the function called for
/// your event handler will NOT have the correct this pointer (at least in
/// Visual C++... ), so you will need to pass a reference to yourself and use
/// that instead. Yeah, its a hack. See the wxBoolOption for more information.
///
//////////////////////////////////////////////////////////////////////////////
class wxOption {
	public:

		/// you should override the following functions:

		//////////////////////////////////////////////////////////////////////////////
		///
		/// \brief This is called when the option is going to be displayed. 
		/// 
		/// You should add your controls to a sizer, and then wait for 
		/// OnWindowDestroy() to be called. If you need to get the value of a control 
		/// (ie, a textbox), then you should keep pointers to them around. 
		/// However, do not delete the controls, since they are owned by the parent 
		/// dialog and will be automatically deleted.
		///
		/// @param sizer is a wxOptionSizer that has two columns that your controls 
		/// should be added to
		///
		/// @param parent is a reference to the parent window that you should pass to your
		/// child controls. 
		///
		//////////////////////////////////////////////////////////////////////////////
		virtual void DoDisplay(wxOptionSizer * sizer, wxWindow * parent) = 0;
        
		//////////////////////////////////////////////////////////////////////////////
		///
		/// \brief When this is called, then you should save out any values stored in
		/// controls, as they will be destroyed soon. 
		///
		//////////////////////////////////////////////////////////////////////////////
		virtual void OnWindowDestroy() = 0;

		//////////////////////////////////////////////////////////////////////////////
		///
		/// \brief Save any values to the config that needs to be saved. Please note
		/// that this will _not_ always be called. It is only called if DoDisplay() 
		/// was called for your option
		///
		//////////////////////////////////////////////////////////////////////////////
		virtual bool DoSave() = 0;

		//////////////////////////////////////////////////////////////////////////////
		///
		/// \brief Enable/disable any controls that need to be enabled/disabled. Don't
		/// bother doing this with static labels, that just seems like it would be 
		/// annoying and useless.
		///
		//////////////////////////////////////////////////////////////////////////////
        virtual bool Enable(bool enable) = 0;

		/// virtual destructor to avoid warnings
		virtual ~wxOption(){};

		//////////////////////////////////////////////////////////////////////////////
		///
		/// \brief This is only here so that some options can create subgroups of the
		/// current wxOptionGroup... its a huge hack, see buttonOption for more 
		/// information
		///
		//////////////////////////////////////////////////////////////////////////////
		virtual void SetParent(wxOptionGroup * parent) { m_parentGroup = parent; };


		/// only in base class, so that DoSave() isn't called unless it has to
		void Display(wxOptionSizer * sizer, wxWindow * parent)
		{
			m_needSave = true;
			DoDisplay(sizer, parent);
		}

		/// only in base class, so that DoSave() isn't called unless it has to
		bool Save() 
		{ 
			bool needSave = m_needSave;
			m_needSave = false;
			if (needSave) 
				return DoSave(); 
			return true; 
		};

	protected:

		wxOption(const wxString& configValueName, const wxString& description ) :
			m_configValueName(configValueName), m_description(description), m_needSave(false) {};

		/// the key value of the option in the config
		wxString m_configValueName;

		/// description of the option
		wxString m_description;

		/// reference to the parent wxOptionGroup of this option. 
		wxOptionGroup * m_parentGroup;

		/// a default constructor if you really need it 
		wxOption() : m_needSave(false) {};

	private:
		bool m_needSave;
};

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implementation for wxOptionPtr
///
/// Level of indirection to encapsulate an wxOption because WX_DECLARE_OBJARRAY
/// and other WX classes/functions want to own everything you send their way,
/// which causes problems when they delete something and we still want to 
/// reuse whatever it was that we passed it -- like Connect()
///
//////////////////////////////////////////////////////////////////////////////
class wxOptionPtr : public wxObject {
	public:

		DECLARE_DYNAMIC_CLASS(wxOptionPtr);

		wxOptionPtr(wxOption * option){
			this->option = option;
		};

		wxOption * option;

	private:
		wxOptionPtr(){}
};



//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements an option that just displays a message.
///
/// TODO: Somehow span both columns... 
///
//////////////////////////////////////////////////////////////////////////////
class nullOption : public wxOption {
	public:
		nullOption( const wxString& description ) : wxOption(wxT(""), description){};

		void DoDisplay(wxOptionSizer * sizer, wxWindow * parent) 
		{
			wxOptionSizerFlags flags;
			flags.Align(wxALIGN_CENTER_VERTICAL);
			flags.ColSpan(2);

			sizer->Insert(sizer->GetChildren().size(), new wxOptionSizerItem(new wxStaticText(parent, wxID_ANY, m_description), flags));
		}

		void OnWindowDestroy() { }
		bool DoSave() { return true; }
		bool Enable(bool enable) { return true;  }
};

enum boolStorage {
	boolStoreNormal,
	boolStoreInverted
};


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements a yes/no checkbox option. Can also control a group of
/// related options. 
///
//////////////////////////////////////////////////////////////////////////////
class wxBoolOption : public wxEvtHandler, public wxOption {
	public:
		wxBoolOption( const wxString& configValueName, const wxString& description, bool defaultValue, boolStorage storeInverted = boolStoreNormal, bool enableIfChecked = true) :
			wxOption(configValueName, description ) , m_enableIfChecked(enableIfChecked), m_storeInverted(storeInverted)
		{ 
			m_defaultValue = wxConfigBase::Get()->Read(configValueName, (long)defaultValue) != 0;
			if (storeInverted == boolStoreInverted)
				m_defaultValue = !m_defaultValue;
		}

		wxBoolOption * Add(wxOption * option);
		void OnCheck(wxCommandEvent& event);
        ~wxBoolOption();
        
		void DoDisplay(wxOptionSizer * sizer, wxWindow * parent);
		
		void OnWindowDestroy();
		bool DoSave();
        bool Enable(bool enable);

		void SetParent(wxOptionGroup * parent);

	private:
        bool ChildEnabled(bool enable);
    
        bool m_enableIfChecked;
		bool m_defaultValue;
		boolStorage m_storeInverted;

		wxCheckBox * m_checkBox;
        Options m_options;
};



//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements a string textcontrol option
///
//////////////////////////////////////////////////////////////////////////////
class wxStringOption : public wxOption {
	public:
		wxStringOption(const wxString& configValueName, const wxString& description, const wxString& defaultValue) :
			wxOption(configValueName, description ) 
			{ m_defaultValue = wxConfigBase::Get()->Read(configValueName, defaultValue); };

		virtual void DoDisplay(wxOptionSizer * sizer, wxWindow * parent) 
		{
			OPTION_ADD_DEFAULT_LABEL;
			sizer->Add(m_textCtrl = new wxTextCtrl(parent, wxID_ANY, m_defaultValue));
		}

		virtual void OnWindowDestroy() { m_defaultValue = m_textCtrl->GetValue(); }
		virtual bool DoSave() { return wxConfigBase::Get()->Write(m_configValueName, m_defaultValue); }
        virtual bool Enable(bool enable) { return m_textCtrl->Enable(enable); }

	protected:
		wxString m_defaultValue;
		wxTextCtrl * m_textCtrl;
		wxStringOption(){}
};


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Derives from wxStringOption, only overrides the display() functions
///
//////////////////////////////////////////////////////////////////////////////
class wxPasswordOption : public wxStringOption {
	public:

		wxPasswordOption(const wxString& configValueName, const wxString& description, const wxString& defaultValue) :
			wxStringOption(configValueName, description, defaultValue){}

		virtual void DoDisplay(wxOptionSizer * sizer, wxWindow * parent) {
			OPTION_ADD_DEFAULT_LABEL;
			sizer->Add(m_textCtrl = new wxTextCtrl(parent, wxID_ANY, m_defaultValue, wxDefaultPosition, wxDefaultSize, wxTE_PASSWORD));
		}
};

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements an integer option. 
///
/// TODO: Make minValue and maxValue actually do something. Not used right now. 
///
//////////////////////////////////////////////////////////////////////////////
class wxIntOption : public wxOption {
	public:
		wxIntOption(const wxString& configValueName, const wxString& description, 
			long defaultValue, bool hasMinAndMax = false, long minValue = 0, long maxValue = 0) :
			wxOption(configValueName, description ) 
			{ m_defaultValue = wxConfigBase::Get()->Read(configValueName, defaultValue); }

		virtual void DoDisplay(wxOptionSizer * sizer, wxWindow * parent) 
		{
			OPTION_ADD_DEFAULT_LABEL;
			sizer->Add(m_textCtrl = new wxTextCtrl(parent, wxID_ANY, wxString::Format(wxT("%ld"),m_defaultValue), 
				wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC)));
		}

		virtual void OnWindowDestroy() {	m_textCtrl->GetValue().ToLong(&m_defaultValue); };
		bool DoSave() { return wxConfigBase::Get()->Write(m_configValueName, m_defaultValue); }
        virtual bool Enable(bool enable) { return m_textCtrl->Enable(enable); }

	protected:
		long m_defaultValue;
		wxTextCtrl * m_textCtrl;
};

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements a double option. TODO: need to implement min/max
///
//////////////////////////////////////////////////////////////////////////////
class wxDoubleOption : public wxOption {
	public:
		wxDoubleOption(const wxString& configValueName, const wxString& description, 
			double defaultValue, bool hasMinAndMax = false, double minValue = 0, double maxValue = 0) :
			wxOption(configValueName, description ) 
			{ wxConfigBase::Get()->Read(configValueName, &m_defaultValue, defaultValue); }

		virtual void DoDisplay(wxOptionSizer * sizer, wxWindow * parent) 
		{
			OPTION_ADD_DEFAULT_LABEL;
			sizer->Add(m_textCtrl = new wxTextCtrl(parent, wxID_ANY, wxString::Format(wxT("%lf"),m_defaultValue), 
				wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC)));
		}

		virtual void OnWindowDestroy() { m_textCtrl->GetValue().ToDouble(&m_defaultValue); };
		bool DoSave() { return wxConfigBase::Get()->Write(m_configValueName, m_defaultValue); }
        virtual bool Enable(bool enable) { return m_textCtrl->Enable(enable); }

	protected:
		double m_defaultValue;
		wxTextCtrl * m_textCtrl;
};



//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements an integer slider option
///
//////////////////////////////////////////////////////////////////////////////
class wxSliderIntOption : public wxOption {
	public:
		wxSliderIntOption(const wxString& configValueName, const wxString& description, 
			long defaultValue, long minValue = 0, long maxValue = 0) :
			wxOption(configValueName, description ), m_minValue(minValue), m_maxValue(maxValue) 
			{ m_defaultValue = wxConfigBase::Get()->Read(configValueName, defaultValue); }

		virtual void DoDisplay(wxOptionSizer * sizer, wxWindow * parent) 
		{
			OPTION_ADD_DEFAULT_LABEL;
			sizer->Add(m_slider = new wxSlider(parent, wxID_ANY, m_defaultValue, m_minValue, m_maxValue));
		}

		virtual void OnWindowDestroy() { m_defaultValue = m_slider->GetValue(); }
		bool DoSave() { return wxConfigBase::Get()->Write(m_configValueName, m_defaultValue); }
        virtual bool Enable(bool enable) { return m_slider->Enable(enable); }

	protected:
		long m_defaultValue;
		long m_minValue;
		long m_maxValue;
		wxSlider * m_slider;
};


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements a double slider option
///
//////////////////////////////////////////////////////////////////////////////
class wxSliderDoubleOption : public wxOption {
	public:
		// precision specifies how many digits of precision you want to keep when using the slider
		wxSliderDoubleOption(const wxString& configValueName, const wxString& description, 
			double defaultValue, double minValue = 0, double maxValue = 0, long precision = 2) :
			wxOption(configValueName, description ), m_minValue(minValue), m_maxValue(maxValue)
		{ 
			wxConfigBase::Get()->Read(configValueName, &m_defaultValue, defaultValue); 
			p_factor = 10.0;
			for (int i = 1; i < precision; i++)
				p_factor *= 10.0;
		}

		virtual void DoDisplay(wxOptionSizer * sizer, wxWindow * parent) 
		{
			OPTION_ADD_DEFAULT_LABEL;
			sizer->Add(m_slider = new wxSlider(parent, wxID_ANY, m_defaultValue * p_factor, m_minValue * p_factor, m_maxValue * p_factor));
		}

		virtual void OnWindowDestroy() {
			m_defaultValue = ((double)m_slider->GetValue())/p_factor;
		}

		bool DoSave() { return wxConfigBase::Get()->Write(m_configValueName, m_defaultValue); }
        virtual bool Enable(bool enable) { return m_slider->Enable(enable); }

	protected:
		double p_factor;
		double m_defaultValue;
		double m_minValue;
		double m_maxValue;
		wxSlider * m_slider;
};



//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements a string option with an added browse button
///
//////////////////////////////////////////////////////////////////////////////
class wxFileOption : public wxEvtHandler, public wxStringOption {
	public:
		wxFileOption(const wxString& configValueName, const wxString& description, const wxString& defaultValue,
			const wxString& wildcards, const wxString& browseMsg) :
			wxStringOption(configValueName, description, defaultValue ), 
			m_browseMsg(browseMsg), m_wildcards(wildcards)
			{}

		virtual void DoDisplay(wxOptionSizer * sizer, wxWindow * parent);
		void OnBrowse(wxCommandEvent& event);
        bool Enable(bool enable) { return m_textCtrl->Enable(enable) && m_browseButton->Enable(enable); }

	private:
		wxButton * m_browseButton;
		wxString m_browseMsg;
		wxString m_wildcards;
};

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements a string option with an added browse button
///
//////////////////////////////////////////////////////////////////////////////
class wxDirectoryOption : public wxEvtHandler, public wxStringOption {
	public:
		wxDirectoryOption(const wxString& configValueName, const wxString& description, const wxString& defaultValue, 
			const wxString& browseMsg) :
			wxStringOption(configValueName, description, defaultValue ), m_browseMsg(browseMsg)
			{}

		virtual void DoDisplay(wxOptionSizer * sizer, wxWindow * parent);
		void OnBrowse(wxCommandEvent& event);
        bool Enable(bool enable) { return m_textCtrl->Enable(enable) && m_browseButton->Enable(enable); }

	private:
		wxButton * m_browseButton;
		wxString m_browseMsg;
};

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements a combo box option.
///
/// If saveIndex is true, then it saves/reads the index of the value, instead
/// of the item itself. 
///
//////////////////////////////////////////////////////////////////////////////
class wxComboBoxOption : public wxOption {
	public:

		wxComboBoxOption(const wxString& configValueName, const wxString& description, const wxString& defaultValue, bool saveIndex) :
			wxOption(configValueName, description ), 
			m_hasDefault(false), m_defaultValue(defaultValue), m_originalDefaultValue(defaultValue),  
			m_comboBox(NULL) , m_saveIndex(saveIndex)
        { }
        
		// only use one of these for an option, do not mix them!
		wxComboBoxOption* Add(const wxString& item);
		wxComboBoxOption* Add(long index, const wxString& item);

        virtual void DoDisplay(wxOptionSizer * sizer, wxWindow * parent);
        
        virtual void OnWindowDestroy() { m_defaultValue = m_comboBox->GetValue(); m_comboBox = NULL; }
        virtual bool Enable(bool enable) { return m_comboBox->Enable(enable); }
		bool DoSave();

	protected:
		bool m_hasDefault;
        wxString m_defaultValue;
		wxString m_originalDefaultValue;

		// these are always the same length... m_indexes[i] contains the index for m_items[i]
		// however, m_indexes is ignored if m_saveIndex is false 
		wxArrayString m_items;
		wxArrayInt m_indexes;

		wxComboBox * m_comboBox;
		bool m_saveIndex;
}; 


#endif
