/*
 *  wxOptionsDialog.cpp
 *
 *  Copyright (c) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// This defines the implementation for a generic options dialog box
///
/// This is part of a generic set of classes designed so that it can be used 
/// with any project, project-specific options should be added 
/// to OptionsLaunch.cpp
///
//////////////////////////////////////////////////////////////////////////////


#include <wx/wx.h>
#include <wx/sizer.h>

#include "wxOptionsDialog.h"

// probably need to define these magic constants at runtime somehow
#define OPTIONS_MARGIN 15
#define OPTIONS_BTN_X 140
#define OPTIONS_BTN_Y 40

BEGIN_EVENT_TABLE(wxOptionsDialog, wxDialog)
	EVT_BUTTON(wxID_SAVE, wxOptionsDialog::OnSave)
	EVT_BUTTON(wxID_BACKWARD, wxOptionsDialog::OnBack)
END_EVENT_TABLE()


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Constructor for options dialog
///
///	Sets up the initial stuff needed, constructs the menus/options
///
//////////////////////////////////////////////////////////////////////////////
wxOptionsDialog::wxOptionsDialog(wxWindow * parent, wxOptionGroup * rootOptionGroup) : wxDialog(parent, -1, wxString(wxT("Options")), wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxDEFAULT_DIALOG_STYLE)
{
	// initialize needed variables
	m_innerSizer = NULL;
	
	m_currentOptionGroup = m_rootOptionGroup = rootOptionGroup;

	// ok, do the initial setup required
	m_outerSizer = new wxBoxSizer(wxVERTICAL);
	m_outerSizer->AddSpacer(OPTIONS_MARGIN);

	// render the top level menu
	RenderOptionMenu(false);

	m_outerSizer->AddSpacer(OPTIONS_MARGIN);

	// add the controlling buttons.. need to determine if theres a more intuitive way to do this
	wxBoxSizer * botSizer = new wxBoxSizer(wxHORIZONTAL);
	
	m_backButton = new wxButton(this, wxID_BACKWARD, _("&Back"));
	m_backButton->Disable();

	botSizer->Add(m_backButton, 1, wxSHAPED | wxALIGN_CENTER);
	botSizer->Add(new wxButton(this, wxID_SAVE, _("&Save")), 1, wxSHAPED | wxALIGN_CENTER);
	m_outerSizer->Add(botSizer, 1, wxEXPAND);

	m_outerSizer->AddSpacer(OPTIONS_MARGIN);

    // and... done
	SetSizerAndFit(m_outerSizer);
}


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Renders the current menu or options in the innerSizer
///
///	This assumes that some other function set the current menu group or 
/// option set correctly (and that its a valid menu group or option set). 
/// 
/// \param doLayoutAndFit	Automatically perform a layout/fit when done
///
//////////////////////////////////////////////////////////////////////////////
void wxOptionsDialog::RenderOptionMenu(bool doLayoutAndFit)
{
	wxSizer * szr = NULL;
	size_t i;

	SetTitle(m_currentOptionGroup->GetName());

	// if we're at a menu group, then display the group
	if (m_currentOptionGroup->HasGroups()){

		const wxOptionGroups * groups = m_currentOptionGroup->GetGroups();
		wxGridSizer * sizer;
		
		szr = sizer = new wxGridSizer(0, 2, OPTIONS_MARGIN, OPTIONS_MARGIN);
		
		for (i = 0; i < groups->Count(); i++)
		{
			wxButton * btn = new wxButton(this, wxID_ANY, groups->Item(i).GetName());

			btn->SetMinSize(wxSize(OPTIONS_BTN_X, OPTIONS_BTN_Y));
			
			// set up a dynamic event handler, pass it the group to make life way easier
			Connect(btn->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
				wxCommandEventHandler(wxOptionsDialog::OnMenuClick), 
				wxDynamicCast(new wxOptionGroupPtr(&(groups->Item(i))),wxObject));

			if (i%2)
				sizer->Add(btn,1, wxALIGN_LEFT);
			else
				sizer->Add(btn, 1, wxALIGN_RIGHT);
		}

		if (i == 1)
			sizer->AddSpacer(1);


	} else if (m_currentOptionGroup->HasOptions()){

		// otherwise, determine how to display the desired options
		const Options * options = m_currentOptionGroup->GetOptions();
		wxOptionSizer * sizer;

		szr = sizer = new wxOptionSizer(2, OPTIONS_MARGIN/2, OPTIONS_MARGIN);

		for (i = 0; i < options->Count(); i++)
			options->Item(i).option->Display( sizer, this);

		
		// its a reasonable thought that any labels being used 
		// at the top level of the sizer should be resized so the 
		// dialog doesn't get TOO big

		wxSizerItemList list = sizer->GetChildren();
#if wxCHECK_VERSION(2, 8, 0)
		for (wxSizerItemList::iterator iter = list.begin(); iter != list.end(); ++iter)
		{
			wxStaticText * item = wxDynamicCast((*iter)->GetWindow(), wxStaticText);
#else
		for (wxSizerItemList::Node *node = list.GetFirst(); node; node = node->GetNext())
		{
			wxStaticText * item = wxDynamicCast(((node->GetData()))->GetWindow(), wxStaticText);
#endif
			if (item){
				wxSize sz = GetClientSize();
				item->Wrap((sz.GetWidth() * 3)/ 4);
			}
		}
		

	} else {
		wxFAIL_MSG(wxT("No options or groups to display!"));
	}

	if (szr){

		// add a border to our sizer
		wxBoxSizer * tmp = new wxBoxSizer(wxHORIZONTAL);
		tmp->Add(szr,1, wxLEFT | wxRIGHT, OPTIONS_MARGIN);
		szr = tmp;


		if (!m_innerSizer){
			// this should only occur at the constructor
			m_innerSizer = szr;
			m_outerSizer->Add(szr);
		}else{

			// hide item
			m_outerSizer->Show(m_innerSizer, false);

			// replace item
#if wxCHECK_VERSION(2,9,0)
			m_outerSizer->GetItem(m_innerSizer)->AssignSizer(szr);
#else
			m_outerSizer->GetItem(m_innerSizer)->SetSizer(szr);
			delete m_innerSizer;
#endif
			m_innerSizer = szr;

			// show the item once again
			m_outerSizer->Show(m_innerSizer);
		}
	}

	// layout and fit
	if (doLayoutAndFit){
		m_outerSizer->Fit(this);
		Layout();
	}
}


//////////////////////////////////////////////////////////////////////////////
///
/// \brief This is called anytime one of the menu buttons is clicked. It grabs
/// the wxOptionGroupPtr we handed to Connect() and creates a new menu based on
/// that information 
///
//////////////////////////////////////////////////////////////////////////////
void wxOptionsDialog::OnMenuClick(wxCommandEvent& event)
{
	m_currentOptionGroup = wxDynamicCast ( event.m_callbackUserData, wxOptionGroupPtr )->group;
	m_backButton->Enable();
	RenderOptionMenu();
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief This MUST be called anytime a control is going to be destroyed, so
/// that the associated options can grab the option data from their controls. 
/// Of course, you should call this before calling save also. 
///
//////////////////////////////////////////////////////////////////////////////
void wxOptionsDialog::StoreOptions()
{
	const Options * options = m_currentOptionGroup->HasOptions() ? m_currentOptionGroup->GetOptions() : NULL;

	// tell options that they're about to die
	if (options)
		for (size_t i = 0; i < options->Count(); i++)
			options->Item(i).option->OnWindowDestroy();
}


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Called when the save button is clicked
///
//////////////////////////////////////////////////////////////////////////////
void wxOptionsDialog::OnSave(wxCommandEvent& event)
{
	// THE WORLD IS ENDING! SAVE YOURSELVES!!!
	StoreOptions();
	m_rootOptionGroup->Save();

	// we're done, let the caller destroy us
	EndModal(wxID_SAVE);
}


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Called when the back button is clicked
///
//////////////////////////////////////////////////////////////////////////////
void wxOptionsDialog::OnBack(wxCommandEvent& event)
{

	StoreOptions();
 
	m_currentOptionGroup = m_currentOptionGroup->GetParent();

	// OH NOES! ASSERT AN ERROR HERE!
	wxASSERT_MSG(m_currentOptionGroup != NULL, wxT("This should not be NULL, ever. Oops... "));

	// ensure the parent of the new current item isn't null
	if (m_currentOptionGroup->GetParent() == NULL)
		m_backButton->Disable();
		
	// ok, we went up a level, render this menu
	RenderOptionMenu();


}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Destructor cleans up
///
//////////////////////////////////////////////////////////////////////////////
wxOptionsDialog::~wxOptionsDialog()
{ 	
	delete m_rootOptionGroup;
}

