/*
 *  wxOptionsDialog.h
 *
 *  Copyright (c) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// This defines a generic options dialog
///
/// This is part of a generic set of classes designed so that it can be used 
/// with any project, project-specific options should be added 
/// to OptionsLaunch.cpp
///
//////////////////////////////////////////////////////////////////////////////

#ifndef OPTIONSDIALOG_H
#define OPTIONSDIALOG_H

#include "wxOptionGroup.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements the options dialog box
///
/// The goal here is twofold: to be friendly for small screens (640x480, 
/// 800x480 and the like), and to be relatively easy to change the options
/// around without having to create new dialogs for each new category (so if
/// you wanted, you could create a more traditional option dialog out of this)
///
/// Something that CAN be implemented (and probably will), is since the 
/// actual options themselves are defined generically, you could define
/// different styles of dialogs to be shown to the user and switch them on the
/// fly quite easily. 
///
//////////////////////////////////////////////////////////////////////////////
class wxOptionsDialog : public wxDialog 
{
	public:
		wxOptionsDialog(wxWindow * parent, wxOptionGroup * rootOptionGroup);
		~wxOptionsDialog();

		void OnMenuClick(wxCommandEvent& event);

	private:

		wxOptionsDialog();
		
		void OnSave(wxCommandEvent& event);
		void OnBack(wxCommandEvent& event);

		void RenderOptionMenu(bool doLayoutAndFit = true);
		void StoreOptions();

		wxBoxSizer * m_outerSizer;
		wxSizer * m_innerSizer;

		wxButton * m_backButton;

		wxOptionGroup * m_rootOptionGroup;
		wxOptionGroup * m_currentOptionGroup;

		DECLARE_EVENT_TABLE()
};

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements an option that has a button, which brings you to 
/// another set of options. Like an optionGroup, but can be present inside a
/// set of options. It should be noted that this option MUST be used with an 
/// wxOptionsDialog window, since it cheats in regards to the whole 
/// object-oriented paradigm... heh
///
/// The way this works is it hides an wxOptionGroup inside of it, and redirects
/// calls to the wxOptionGroup when necessary. The key cheating part is when it
/// simulates a button press on the parent dialog by calling OnMenuClick with
/// a pointer to the group that is hidden by this group. 
///
//////////////////////////////////////////////////////////////////////////////
class buttonOption : public wxEvtHandler, public wxOption {
	public:
		buttonOption( const wxString& name ) : wxOption(wxT(""), name){
			m_group = new wxOptionGroup(name);
		};
		
        // allows chaining
		buttonOption * Add(wxOption * option){ m_group->Add(option); return this; };
        
		void DoDisplay(wxOptionSizer * sizer, wxWindow * parent){
			sizer->AddStretchSpacer();
			//sizer->Add(0, 0, 1);
			sizer->Add(m_button = new wxButton(parent, wxID_ANY, m_description));

			m_button->Connect(m_button->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
				wxCommandEventHandler(buttonOption::OnClick), 
				wxDynamicCast(new wxOptionPtr(this),wxObject));

			m_parentDlg = wxDynamicCast(parent, wxOptionsDialog);
		}

		// this is a nasty hack
		void OnClick(wxCommandEvent& event ){
			buttonOption * _this = (buttonOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;

			// pretend to raise a button click event on the dialog
			wxCommandEvent newEvent;
			newEvent.m_callbackUserData = wxDynamicCast(new wxOptionGroupPtr(_this->m_group), wxObject);

			if (_this->m_parentDlg)
				_this->m_parentDlg->OnMenuClick(newEvent);
		}
		
		bool DoSave(){
			const Options * options = m_group->GetOptions();
			bool ret = true;
			for (size_t i = 0; i < options->Count(); i++)
				ret = ret && options->Item(i).option->Save();

			return ret;
		};

		bool Enable(bool enable){ return m_button->Enable(enable); };
		void OnWindowDestroy() { };
		void SetParent(wxOptionGroup * parent){ m_group->SetParent(parent); };

		~buttonOption(){ delete m_group; };

	private:
    
		wxButton * m_button;
        wxOptionGroup * m_group;
		wxOptionsDialog * m_parentDlg;
};

#define DEF_SUB_OPTION_GROUP(text) \
	DEF_GROWABLE_THING(new buttonOption(text))

#define END_SUB_OPTION_GROUP END_GROWABLE_THING

#endif

